from fabric.api import env, run, cd

env.hosts = ['root@lysithea.proxience.com']

def _deploy(directory=''):
    with cd(directory):
        run('git pull')
        run('/root/bin/redmine_git_fetch')
        run('./manage.py compilemessages -l fr')
        run('/etc/init.d/apache2 restart')
        with cd('../docs'):
            run('make html')

def deploy_test():
    _deploy('/var/local/ishtar/ishtar')

def deploy_prod():
    _deploy('/var/local/ishtar-test/ishtar')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2013-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

import hashlib
import random

from django.conf import settings
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _, ugettext
from django.template.defaultfilters import slugify


def get_cache(cls, extra_args=[]):
    cache_key = u"{}-{}-{}".format(
        settings.PROJECT_SLUG, cls._meta.app_label, cls.__name__)
    for arg in extra_args:
        if not arg:
            cache_key += '-0'
        else:
            if type(arg) == dict:
                cache_key += '-' + "_".join([unicode(arg[k]) for k in arg])
            elif type(arg) in (list, tuple):
                cache_key += '-' + "_".join([unicode(v) for v in arg])
            else:
                cache_key += '-' + unicode(arg)
    cache_key = slugify(cache_key)
    if not cache_key.endswith('_current_keys') \
            and hasattr(cls, '_add_cache_key_to_refresh'):
        cls._add_cache_key_to_refresh(extra_args)
    if len(cache_key) >= 250:
        m = hashlib.md5()
        m.update(cache_key)
        cache_key = m.hexdigest()
    return cache_key, cache.get(cache_key)


def cached_label_changed(sender, **kwargs):
    if not kwargs.get('instance'):
        return
    instance = kwargs.get('instance')
    lbl = instance._generate_cached_label()
    if lbl != instance.cached_label:
        instance.cached_label = lbl
        instance.save()

SHORTIFY_STR = ugettext(" (...)")


def shortify(lbl, number=20):
    if len(lbl) <= number:
        return lbl
    return lbl[:number - len(SHORTIFY_STR)] + SHORTIFY_STR


def mode(array):
    most = max(list(map(array.count, array)))
    return list(set(filter(lambda x: array.count(x) == most, array)))


def _get_image_link(item):
    # manage missing images
    if not item.thumbnail or not item.thumbnail.url:
        return ""
    return mark_safe(u"""
    <div class="welcome-image">
        <img src="{}"/><br/>
        <em>{} - {}</em>
        <a href="#" onclick="load_window(\'{}\')">
          <i class="fa fa-info-circle" aria-hidden="true"></i>
        </a>
        <a href="." title="{}">
            <i class="fa fa-random" aria-hidden="true"></i>
        </a><br/>
    </div>""".format(
        item.thumbnail.url,
        unicode(item.__class__._meta.verbose_name),
        unicode(item),
        reverse(item.SHOW_URL, args=[item.pk, '']),
        unicode(_(u"Load another random image?"))))


def get_random_item_image_link(request):
    from archaeological_operations.models import Operation
    from archaeological_context_records.models import ContextRecord
    from archaeological_finds.models import Find

    ope_image_nb, cr_image_nb, find_image_nb = 0, 0, 0
    if request.user.has_perm('archaeological_operations.view_operation',
                             Operation):
        ope_image_nb = Operation.objects.filter(
            thumbnail__isnull=False).count()
    if request.user.has_perm(
            'archaeological_context_records.view_contextrecord',
            ContextRecord):
        cr_image_nb = ContextRecord.objects.filter(
            thumbnail__isnull=False).count()
    if request.user.has_perm('archaeological_finds.view_find',
                             Find):
        find_image_nb = Find.objects.filter(
            thumbnail__isnull=False).count()

    image_total = ope_image_nb + cr_image_nb + find_image_nb
    if not image_total:
        return ''

    image_nb = random.randint(0, image_total - 1)
    if image_nb >= 0 and image_nb < ope_image_nb:
        return _get_image_link(
            Operation.objects.filter(thumbnail__isnull=False).all()[image_nb])
    if image_nb >= ope_image_nb and image_nb < (cr_image_nb + ope_image_nb):
        return _get_image_link(
            ContextRecord.objects.filter(thumbnail__isnull=False).all()[
                image_nb - ope_image_nb])
    if image_nb >= (cr_image_nb + ope_image_nb):
        return _get_image_link(
            Find.objects.filter(thumbnail__isnull=False).all()[
                image_nb - ope_image_nb - cr_image_nb])
    # should never happen except in case of deletion during the excution
    return ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2014  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from archaeological_files.wizards import FileWizard as BaseFileWizard
from archaeological_files import models


class FileWizard(BaseFileWizard):
    parcel_step_key = 'parcelspdl-'
    town_step_keys = ['preventiveplanning-', 'researchaddress-']
    town_input_id = 'town'
    towns_formset = False
    multi_towns = True
    wizard_templates = {
        'generalcontractor-%(url_name)s':
            'ishtar/wizard/wizard_generalcontractor.html',
        'planningservice-%(url_name)s':
            'ishtar/wizard/wizard_planningservice.html',
        'instruction-%(url_name)s':
            'ishtar/wizard/wizard_instruction.html',
        'preventiveplanning-%(url_name)s':
            'ishtar/wizard/wizard_preventiveplanning.html',
    }
    wizard_confirm = 'ishtar/wizard/file_confirm_wizard.html'

    def get_current_year(self):
        general_form_key = 'general-' + self.url_name
        return self.session_get_value(general_form_key, 'year')

    def get_form_kwargs(self, *args, **kwargs):
        returned = super(FileWizard, self).get_form_kwargs(*args, **kwargs)
        if args and args[0].startswith('generalcontractor-'):
            if 'status' in self.request.GET:
                returned['status'] = self.request.GET['status']
        if args and args[0].startswith('instruction-'):
            returned['year'] = self.get_current_year()
            returned['saisine_type'] = self.get_saisine_type()
            returned['reception_date'] = \
                self.session_get_value(
                    'general-' + self.url_name, 'reception_date')
        return returned

    def get_saisine_type(self):
        try:
            idx = int(
                self.session_get_value(
                    'preventivetype-' + self.url_name, 'saisine_type'))
            return models.SaisineType.objects.get(pk=idx)
        except (TypeError, ValueError, models.PermitType.DoesNotExist):
            pass

    def get_context_data(self, form, **kwargs):
        context = super(FileWizard, self).get_context_data(form)
        formplanning = "planningservice-" + self.url_name
        forminstruction = "instruction-" + self.url_name
        formfinal = "final-" + self.url_name
        if self.steps.current == formplanning:
            try:
                idx = int(
                    self.session_get_value(
                        'preventivetype-' + self.url_name, 'permit_type'))
                permit_type = models.PermitType.objects.get(pk=idx)
                context['permit_type'] = unicode(permit_type)
                context['permit_type_code'] = unicode(permit_type.txt_idx)
            except (TypeError, ValueError, models.PermitType.DoesNotExist):
                pass
        elif self.steps.current == forminstruction:
            saisine_type = self.get_saisine_type()
            if saisine_type:
                context['saisine_type'] = unicode(saisine_type)
                context['saisine_type_delay'] = saisine_type.delay or 0
        elif self.steps.current == formfinal:
            if self.steps.current.endswith('creation'):  # creation only
                parcels = []
                parcel_step_key = self.parcel_step_key + self.url_name

                parcel_numbers = self.session_get_value(
                    parcel_step_key, 'parcel_number', multi=True) or []
                sections = self.session_get_value(
                    parcel_step_key, 'section', multi=True) or []
                towns = self.session_get_value(
                    parcel_step_key, 'town', multi=True) or []
                for idx, parcel_number in enumerate(parcel_numbers):
                    if not parcel_number:
                        continue
                    parcels.append({
                        'town': towns[idx],
                        'section': sections[idx],
                        'parcel_number': parcel_number})
                context['similar_files'] = models.File.similar_files(parcels)
            else:  # edition only
                try:
                    numeric_reference = int(
                        self.session_get_value(
                            'instruction-' + self.url_name,
                            'numeric_reference'))

                    q = models.File.objects.filter(
                        numeric_reference=numeric_reference,
                        year=self.get_current_year()).exclude(
                        pk=self.get_current_object().pk)
                    context['numeric_reference_files'] = q.all()
                except ValueError:
                    pass

        return context


class FileModificationWizard(FileWizard):
    modification = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf.urls.defaults import *

from ishtar_common.wizards import check_rights
import views
import models

# be carreful: each check_rights must be relevant with ishtar_menu

# forms
urlpatterns = patterns(
    '',
    url(r'operation_administrativeactop_search/(?P<step>.+)?$',
        check_rights(['change_administrativeact'])(
            views.operation_administrativeactop_search_wizard),
        name='operation_administrativeactop_search'),
    url(r'operation_administrativeactop/(?P<step>.+)?$',
        check_rights(['change_administrativeact'])(
            views.operation_administrativeactop_wizard),
        name='operation_administrativeactop'),
    url(r'operation_administrativeactop_modification/(?P<step>.+)?$',
        check_rights(['change_administrativeact'])(
            views.operation_administrativeactop_modification_wizard),
        name='operation_administrativeactop_modification'),
    url(r'operation_administrativeactop_modify/(?P<pk>.+)/$',
        views.operation_administrativeactop_modify,
        name='operation_administrativeactop_modify'),
    url(r'operation_administrativeactop_deletion/(?P<step>.+)?$',
        check_rights(['change_administrativeact'])(
            views.operation_administrativeactop_deletion_wizard),
        name='operation_administrativeactop_deletion'),
    url(r'operation_source_search/(?P<step>.+)?$',
        check_rights(['view_operation', 'view_own_operation'])(
            views.operation_source_search_wizard),
        name='operation_source_search'),
    url(r'operation_source_creation/(?P<step>.+)?$',
        check_rights(['change_operation', 'change_own_operation'])(
            views.operation_source_creation_wizard),
        name='operation_source_creation'),
    url(r'operation_source_modification/(?P<step>.+)?$',
        check_rights(['change_operation', 'change_own_operation'])(
            views.operation_source_modification_wizard),
        name='operation_source_modification'),
    url(r'operation_source_modify/(?P<pk>.+)/$',
        views.operation_source_modify, name='operation_source_modify'),
    url(r'operation_source_deletion/(?P<step>.+)?$',
        check_rights(['change_operation', 'change_own_operation'])(
            views.operation_source_deletion_wizard),
        name='operation_source_deletion'),
    url(r'operation_search/(?P<step>.+)?$',
        check_rights(['view_operation', 'view_own_operation'])(
            views.operation_search_wizard), name='operation_search'),
    url(r'operation_creation/(?P<step>.+)?$',
        check_rights(['add_operation'])(views.operation_creation_wizard),
        name='operation_creation'),
    url(r'operation_add/(?P<file_id>\d+)$',
        views.operation_add, name='operation_add'),
    url(r'operation_modification/(?P<step>.+)?$',
        check_rights(['change_operation', 'change_own_operation'])(
            views.operation_modification_wizard),
        name='operation_modification'),
    url(r'operation_modify/(?P<pk>.+)/$',
        views.operation_modify, name='operation_modify'),
    url(r'operation_closing/(?P<step>.+)?$',
        check_rights(['change_operation'])(
            views.operation_closing_wizard), name='operation_closing'),
    url(r'operation_deletion/(?P<step>.+)?$',
        check_rights(['change_operation', 'change_own_operation'])(
            views.operation_deletion_wizard), name='operation_deletion'),
    url(r'administrativact_register/(?P<step>.+)?$',
        check_rights(['view_administrativeact', 'view_own_administrativeact'])(
            views.administrativact_register_wizard),
        name='administrativact_register'),
)

urlpatterns += patterns(
    'archaeological_operations.views',
    url(r'autocomplete-operation/$', 'autocomplete_operation',
        name='autocomplete-operation'),
    url(r'get-operation/own/(?P<type>.+)?$',
        'get_operation', name='get-own-operation',
        kwargs={'force_own': True}),
    url(r'get-operation/(?P<type>.+)?$', 'get_operation',
        name='get-operation'),
    url(r'get-operation-full/own/(?P<type>.+)?$',
        'get_operation', name='get-own-operation-full',
        kwargs={'full': True, 'force_own': True}),
    url(r'get-operation-full/(?P<type>.+)?$', 'get_operation',
        name='get-operation-full', kwargs={'full': True}),
    url(r'get-operation-shortcut/(?P<type>.+)?$',
        'get_operation', name='get-operation-shortcut',
        kwargs={'full': 'shortcut'}),
    url(r'get-available-operation-code/(?P<year>.+)?$',
        'get_available_operation_code', name='get_available_operation_code'),
    url(r'revert-operation/(?P<pk>.+)/(?P<date>.+)$',
        'revert_operation', name='revert-operation'),
    url(r'show-operation(?:/(?P<pk>.+))?/(?P<type>.+)?$',
        'show_operation', name=models.Operation.SHOW_URL),
    url(r'show-historized-operation/(?P<pk>.+)?/(?P<date>.+)?$',
        'show_operation', name='show-historized-operation'),
    url(r'get-administrativeactop/(?P<type>.+)?$',
        'get_administrativeactop', name='get-administrativeactop'),
    url(r'get-administrativeact/(?P<type>.+)?$',
        'get_administrativeact', name='get-administrativeact'),
    url(r'get-administrativeact-full/(?P<type>.+)?$',
        'get_administrativeact', name='get-administrativeact-full',
        kwargs={'full': True}),
    url(r'show-administrativeact(?:/(?P<pk>.+))?/(?P<type>.+)?$',
        'show_administrativeact', name='show-administrativeact'),
    # allow specialization for operations
    url(r'show-administrativeact(?:/(?P<pk>.+))?/(?P<type>.+)?$',
        'show_administrativeact', name='show-administrativeactop'),
    # allow specialization for files
    url(r'show-administrativeact(?:/(?P<pk>.+))?/(?P<type>.+)?$',
        'show_administrativeact', name='show-administrativeactfile'),
    url(r'generatedoc-administrativeactop/(?P<pk>.+)?/(?P<template_pk>.+)?$',
        'generatedoc_administrativeactop',
        name='generatedoc-administrativeactop'),
    url(r'show-operationsource(?:/(?P<pk>.+))?/(?P<type>.+)?$',
        'show_operationsource', name=models.OperationSource.SHOW_URL),
    url(r'get-operationsource/(?P<type>.+)?$',
        'get_operationsource', name='get-operationsource'),
    url(r'get-operationsource-full/(?P<type>.+)?$',
        'get_operationsource', name='get-operationsource-full',
        kwargs={'full': True}),
    url(r'dashboard_operation/$', 'dashboard_operation',
        name='dashboard-operation'),
    url(r'autocomplete-archaeologicalsite/$',
        'autocomplete_archaeologicalsite',
        name='autocomplete-archaeologicalsite'),
    url(r'new-archaeologicalsite/(?:(?P<parent_name>[^/]+)/)?'
        r'(?:(?P<limits>[^/]+)/)?$',
        'new_archaeologicalsite', name='new-archaeologicalsite'),
    url(r'autocomplete-patriarche/$', 'autocomplete_patriarche',
        name='autocomplete-patriarche'),
    url(r'operation_administrativeact_document/$',
        'administrativeactfile_document',
        name='operation-administrativeact-document'),
)



import models


registered_models = {}


def register(model, app=None, manager_name='history'):
    """
    Create historical model for `model` and attach history manager to `model`.

    Keyword arguments:
    app -- App to install historical model into (defaults to model.__module__)
    manager_name -- class attribute name to use for historical manager

    This method should be used as an alternative to attaching an
    `HistoricalManager` instance directly to `model`.
    """
    if not model in registered_models:
        records = models.HistoricalRecords()
        records.manager_name = manager_name
        records.module = ("%s.models" % app) or model.__module__
        records.finalize(model)



import copy
from django.db import models
from django.contrib import admin
from django.contrib.auth.models import User
from django.utils import importlib
from manager import HistoryDescriptor


class HistoricalRecords(object):
    def contribute_to_class(self, cls, name):
        self.manager_name = name
        self.module = cls.__module__
        models.signals.class_prepared.connect(self.finalize, sender=cls)

        def save_without_historical_record(self, *args, **kwargs):
            """Caution! Make sure you know what you're doing before you use this method."""
            self.skip_history_when_saving = True
            ret = self.save(*args, **kwargs)
            del self.skip_history_when_saving
            return ret
        setattr(cls, 'save_without_historical_record', save_without_historical_record)

    def finalize(self, sender, **kwargs):
        history_model = self.create_history_model(sender)
        module = importlib.import_module(self.module)
        setattr(module, history_model.__name__, history_model)

        # The HistoricalRecords object will be discarded,
        # so the signal handlers can't use weak references.
        models.signals.post_save.connect(self.post_save, sender=sender,
                                         weak=False)
        models.signals.post_delete.connect(self.post_delete, sender=sender,
                                           weak=False)

        descriptor = HistoryDescriptor(history_model)
        setattr(sender, self.manager_name, descriptor)
        sender._meta.simple_history_manager_attribute = self.manager_name

    def create_history_model(self, model):
        """
        Creates a historical model to associate with the model provided.
        """
        attrs = {'__module__': self.module}

        fields = self.copy_fields(model)
        attrs.update(fields)
        attrs.update(self.get_extra_fields(model, fields))
        attrs.update(Meta=type('Meta', (), self.get_meta_options(model)))
        name = 'Historical%s' % model._meta.object_name
        return type(name, (models.Model,), attrs)

    def copy_fields(self, model):
        """
        Creates copies of the model's original fields, returning
        a dictionary mapping field name to copied field object.
        """
        fields = {}

        for field in model._meta.fields:
            field = copy.copy(field)
            fk = None

            if isinstance(field, models.AutoField):
                # The historical model gets its own AutoField, so any
                # existing one must be replaced with an IntegerField.
                field.__class__ = models.IntegerField

            if isinstance(field, models.ForeignKey):
                field.__class__ = models.IntegerField
                #ughhhh. open to suggestions here
                try:
                    field.rel = None
                except:
                    pass
                try:
                    field.related = None
                except:
                    pass
                try:
                    field.related_query_name = None
                except:
                    pass
                field.null = True
                field.blank = True
                fk = True
            else:
                fk = False

            # The historical instance should not change creation/modification timestamps.
            field.auto_now = False
            field.auto_now_add = False

            if field.primary_key or field.unique:
                # Unique fields can no longer be guaranteed unique,
                # but they should still be indexed for faster lookups.
                field.primary_key = False
                field._unique = False
                field.db_index = True
                field.serialize = True
            if fk:
                field.name = field.name + "_id"
            fields[field.name] = field

        return fields

    def get_extra_fields(self, model, fields):
        """
        Returns a dictionary of fields that will be added to the historical
        record model, in addition to the ones returned by copy_fields below.
        """
        @models.permalink
        def revert_url(self):
            opts = model._meta
            return ('%s:%s_%s_simple_history' %
                    (admin.site.name, opts.app_label, opts.module_name),
                    [getattr(self, opts.pk.attname), self.history_id])
        def get_instance(self):
            return model(**dict([(k, getattr(self, k)) for k in fields]))

        return {
            'history_id': models.AutoField(primary_key=True),
            'history_date': models.DateTimeField(auto_now_add=True),
            'history_user': models.ForeignKey(User, null=True),
            'history_type': models.CharField(max_length=1, choices=(
                ('+', 'Created'),
                ('~', 'Changed'),
                ('-', 'Deleted'),
            )),
            'history_object': HistoricalObjectDescriptor(model),
            'instance': property(get_instance),
            'revert_url': revert_url,
            '__unicode__': lambda self: u'%s as of %s' % (self.history_object,
                                                          self.history_date)
        }

    def get_meta_options(self, model):
        """
        Returns a dictionary of fields that will be added to
        the Meta inner class of the historical record model.
        """
        return {
            'ordering': ('-history_date', '-history_id'),
        }

    def post_save(self, instance, created, **kwargs):
        if not created and hasattr(instance, 'skip_history_when_saving'):
            return
        if not kwargs.get('raw', False):
            self.create_historical_record(instance, created and '+' or '~')

    def post_delete(self, instance, **kwargs):
        self.create_historical_record(instance, '-')

    def create_historical_record(self, instance, type):
        history_user = getattr(instance, '_history_user', None)
        manager = getattr(instance, self.manager_name)
        attrs = {}
        for field in instance._meta.fields:
            attrs[field.attname] = getattr(instance, field.attname)
        manager.create(history_type=type, history_user=history_user, **attrs)


class HistoricalObjectDescriptor(object):
    def __init__(self, model):
        self.model = model

    def __get__(self, instance, owner):
        values = (getattr(instance, f.attname) for f in self.model._meta.fields)
        return self.model(*values)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from bs4 import BeautifulSoup as Soup

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.cache import cache
from django.core.exceptions import ValidationError
from django.core.files.base import File as DjangoFile
from django.core.urlresolvers import reverse
from django.template.defaultfilters import slugify
from django.test import TestCase
from django.test.client import Client

from ishtar_common import models

"""
from django.conf import settings
import tempfile, datetime
from zipfile import ZipFile, ZIP_DEFLATED

from oook_replace.oook_replace import oook_replace

class OOOGenerationTest(TestCase):
    def testGeneration(self):
        context = {'test_var':u"Testé", 'test_var2':u"",
                   "test_date":datetime.date(2015, 1, 1)}
        tmp = tempfile.TemporaryFile()
        oook_replace("../ishtar_common/tests/test-file.odt", tmp, context)
        inzip = ZipFile(tmp, 'r', ZIP_DEFLATED)
        value = inzip.read('content.xml')
        self.assertTrue(u"Testé" in value or "Test&#233;" in value)
        self.assertTrue("testé 2" not in value and "test&#233; 2" not in value)
        self.assertTrue("2015" in value)
        lg, ct = settings.LANGUAGE_CODE.split('-')
        if lg == 'fr':
            self.assertTrue('janvier' in value)
        if lg == 'en':
            self.assertTrue('january' in value)
"""


def create_superuser():
    username = 'username4277'
    password = 'dcbqj756456!@%'
    user = User.objects.create_superuser(username, "nomail@nomail.com",
                                         password)
    return username, password, user


def create_user():
    username = 'username678'
    password = 'dcbqj756456!@%'
    user = User.objects.create_user(username, email="nomail2@nomail.com")
    user.set_password(password)
    user.save()
    return username, password, user


class WizardTest(object):
    url_name = None
    wizard_name = ''
    steps = None
    condition_dict = None
    form_datas = []

    def setUp(self):
        self.username, self.password, self.user = create_superuser()

    def pre_wizard(self):
        self.client.login(**{'username': self.username,
                             'password': self.password})

    def post_wizard(self):
        pass

    def test_wizard(self):
        url = reverse(self.url_name)
        self.pre_wizard()
        for form_data, ignored in self.form_datas:
            for idx, step in enumerate(self.steps):
                current_step, current_form = step
                if current_step in ignored:
                    continue
                data = {
                    '{}{}-current_step'.format(self.url_name,
                                               self.wizard_name):
                    [current_step],
                }
                if current_step in form_data:
                    d = form_data[current_step]
                    for k in d:
                        data['{}-{}'.format(current_step, k)] = d[k]

                next_idx, next_form = idx + 1, None
                while len(self.steps) > next_idx:
                    if self.steps[idx + 1][0] not in ignored:
                        next_form = self.steps[idx + 1][0]
                        break
                    next_idx = next_idx + 1

                if next_form:
                    try:
                        response = self.client.post(url, data)
                    except ValidationError as e:
                        raise ValidationError(u"Errors: {} on {}.".format(
                            u" - ".join(e.messages), current_step))
                    if "errorlist" in response.content:
                        soup = Soup(response.content)
                        errorlist = soup.findAll(
                            "ul", {"class": "errorlist"})
                        errors = []
                        for li in errorlist:
                            lbl = li.findParent().findParent().findChild().text
                            errors.append(u"{} - {}".format(lbl, li.text))
                        raise ValidationError(u"Errors: {} on {}.".format(
                            u" ".join(errors), current_step))
                    self.assertRedirects(
                        response,
                        '/{}/{}'.format(self.url_name, next_form))
                else:
                    response = self.client.post(url, data, follow=True)
        self.post_wizard()


class CacheTest(TestCase):
    def testAdd(self):
        models.OrganizationType.refresh_cache()
        cached = models.OrganizationType.get_cache('test')
        self.assertEqual(cached, None)
        orga = models.OrganizationType.objects.create(
            txt_idx='test', label='testy')
        cached = models.OrganizationType.get_cache('test')
        self.assertEqual(cached.pk, orga.pk)
        orga.txt_idx = 'testy'
        orga.save()
        cached = models.OrganizationType.get_cache('testy')
        self.assertEqual(cached.pk, orga.pk)

    def testList(self):
        models.OrganizationType.refresh_cache()
        types = models.OrganizationType.get_types()
        # only empty
        self.assertTrue(len(types), 1)
        org = models.OrganizationType.objects.create(
            txt_idx='test', label='testy')
        types = [
            unicode(lbl) for idx, lbl in models.OrganizationType.get_types()]
        self.assertTrue('testy' in types)
        org.delete()
        types = [
            unicode(lbl) for idx, lbl in models.OrganizationType.get_types()]
        self.assertFalse('testy' in types)


class MergeTest(TestCase):
    def setUp(self):
        self.user, created = User.objects.get_or_create(username='username')
        self.organisation_types = \
            models.OrganizationType.create_default_for_test()

        self.person_types = [models.PersonType.objects.create(label='Admin'),
                             models.PersonType.objects.create(label='User')]
        self.author_types = [models.AuthorType.objects.create(label='1'),
                             models.AuthorType.objects.create(label='2')]

        self.company_1 = models.Organization.objects.create(
            history_modifier=self.user, name='Franquin Comp.',
            organization_type=self.organisation_types[0])
        self.person_1 = models.Person.objects.create(
            name='Boule', surname=' ', history_modifier=self.user,
            attached_to=self.company_1)
        self.person_1.person_types.add(self.person_types[0])
        self.author_1_pk = models.Author.objects.create(
            person=self.person_1, author_type=self.author_types[0]).pk

        self.title = models.TitleType.objects.create(label='Test')

        self.company_2 = models.Organization.objects.create(
            history_modifier=self.user, name='Goscinny Corp.',
            organization_type=self.organisation_types[1])
        self.person_2 = models.Person.objects.create(
            name='Bill', history_modifier=self.user, surname='Peyo',
            title=self.title, attached_to=self.company_2)
        self.person_2.person_types.add(self.person_types[1])
        self.author_2_pk = models.Author.objects.create(
            person=self.person_2, author_type=self.author_types[1]).pk
        self.person_3 = models.Person.objects.create(
            name='George', history_modifier=self.user,
            attached_to=self.company_1)

    def testPersonMerge(self):
        self.person_1.merge(self.person_2)
        # preserve existing fields
        self.assertEqual(self.person_1.name, 'Boule')
        # fill missing fields
        self.assertEqual(self.person_1.title, self.title)
        # string field with only spaces is an empty field
        self.assertEqual(self.person_1.surname, 'Peyo')
        # preserve existing foreign key
        self.assertEqual(self.person_1.attached_to, self.company_1)
        # preserve existing many to many
        self.assertTrue(self.person_types[0]
                        in self.person_1.person_types.all())
        # add new many to many
        self.assertTrue(self.person_types[1]
                        in self.person_1.person_types.all())
        # update reverse foreign key association and dont break the existing
        self.assertEqual(models.Author.objects.get(pk=self.author_1_pk).person,
                         self.person_1)
        self.assertEqual(models.Author.objects.get(pk=self.author_2_pk).person,
                         self.person_1)

        self.person_3.merge(self.person_1)
        # manage well empty many to many fields
        self.assertTrue(self.person_types[1]
                        in self.person_3.person_types.all())

    def testPersonMergeCandidate(self):
        init_mc = self.person_1.merge_candidate.count()
        person = models.Person.objects.create(
            name=self.person_1.name,
            surname=self.person_1.surname, history_modifier=self.user,
            attached_to=self.person_1.attached_to)
        self.assertEqual(self.person_1.merge_candidate.count(),
                         init_mc + 1)
        person.archive()
        self.assertEqual(self.person_1.merge_candidate.count(),
                         init_mc)


class ImportTest(TestCase):
    def testDeleteRelated(self):
        town = models.Town.objects.create(name='my-test')
        self.assertEqual(models.Town.objects.filter(name='my-test').count(), 1)

        # create an import, fields are not relevant...
        create_user()
        importer_type = models.ImporterType.objects.create(
            associated_models='ishtar_common.models.Person')
        mcc_operation_file = DjangoFile(file(
            settings.ROOT_PATH +
            '../archaeological_operations/tests/MCC-operations-example.csv',
            'rb'))
        imprt = models.Import.objects.create(
            user=models.IshtarUser.objects.all()[0],
            importer_type=importer_type,
            imported_file=mcc_operation_file)

        town.imports.add(imprt)
        imprt.delete()
        # town should be deleted
        self.assertEqual(models.Town.objects.filter(name='my-test').count(), 0)

    def testKeys(self):
        content_type = ContentType.objects.get_for_model(
            models.OrganizationType)

        # creation
        label = u"Ploufé"
        ot = models.OrganizationType.objects.create(label=label)
        self.assertEqual(models.ItemKey.objects.filter(
                         object_id=ot.pk, key=slugify(label),
                         content_type=content_type).count(), 1)
        label_2 = u"Plif"
        ot_2 = models.OrganizationType.objects.create(label=label_2)
        self.assertEqual(models.ItemKey.objects.filter(
                         object_id=ot_2.pk, key=slugify(label_2),
                         content_type=content_type).count(), 1)

        # replace key
        ot_2.add_key(slugify(label), force=True)
        # one key point to only one item
        self.assertEqual(models.ItemKey.objects.filter(
                         key=slugify(label),
                         content_type=content_type).count(), 1)
        # this key point to the right item
        self.assertEqual(models.ItemKey.objects.filter(
                         object_id=ot_2.pk, key=slugify(label),
                         content_type=content_type).count(), 1)

        # modification
        label_3 = "Yop"
        ot_2.label = label_3
        ot_2.txt_idx = slugify(label_3)
        ot_2.save()
        # old label not referenced anymore
        self.assertEqual(models.ItemKey.objects.filter(
                         object_id=ot_2.pk, key=slugify(label_2),
                         content_type=content_type).count(), 0)
        # # forced key association is always here
        # new key is here
        self.assertEqual(models.ItemKey.objects.filter(
                         object_id=ot_2.pk, key=slugify(label),
                         content_type=content_type).count(), 1)
        self.assertEqual(models.ItemKey.objects.filter(
                         object_id=ot_2.pk, key=slugify(label_3),
                         content_type=content_type).count(), 1)


class IshtarSiteProfileTest(TestCase):
    def testRelevance(self):
        cache.set('default-ishtarsiteprofile-is-current-profile', None,
                  settings.CACHE_TIMEOUT)
        profile = models.get_current_profile()
        default_slug = profile.slug
        profile2 = models.IshtarSiteProfile.objects.create(
            label="Test profile 2", slug='test-profile-2')
        profile2.save()
        # when no profile is the current, activate by default the first created
        self.assertTrue(profile.active and not profile2.active)
        profile2.active = True
        profile2 = profile2.save()
        # only one profile active at a time
        profile = models.IshtarSiteProfile.objects.get(slug=default_slug)
        self.assertTrue(profile2.active and not profile.active)
        # activate find active automatically context records
        self.assertFalse(profile.context_record)
        profile.find = True
        profile = profile.save()
        self.assertTrue(profile.context_record)
        # activate warehouse active automatically context records and finds
        self.assertFalse(profile2.context_record or profile2.find)
        profile2.warehouse = True
        profile2 = profile2.save()
        self.assertTrue(profile2.context_record and profile2.find)

    def testDefaultProfile(self):
        cache.set('default-ishtar_common-IshtarSiteProfile', None,
                  settings.CACHE_TIMEOUT)
        self.assertFalse(models.IshtarSiteProfile.objects.count())
        profile = models.get_current_profile(force=True)
        self.assertTrue(profile)
        self.assertEqual(models.IshtarSiteProfile.objects.count(), 1)

    def testMenuFiltering(self):
        cache.set('default-ishtarsiteprofile-is-current-profile', None,
                  settings.CACHE_TIMEOUT)
        username = 'username4277'
        password = 'dcbqj756456!@%'
        User.objects.create_superuser(username, "nomail@nomail.com",
                                      password)
        c = Client()
        c.login(username=username, password=password)
        response = c.get(reverse('start'))
        self.assertFalse("section-file_management" in response.content)
        profile = models.get_current_profile()
        profile.files = True
        profile.save()
        response = c.get(reverse('start'))
        self.assertTrue("section-file_management" in response.content)

    def testExternalKey(self):
        profile = models.get_current_profile()
        p = models.Person.objects.create(name='plouf', surname=u'Tégada')
        self.assertEqual(p.raw_name, u"PLOUF Tégada")
        profile.person_raw_name = u'{surname|slug} {name}'
        profile.save()
        p.raw_name = ''
        p.save()
        self.assertEqual(p.raw_name, u"tegada plouf")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2015 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Unit tests
"""
import datetime
import json

from django.conf import settings
from django.contrib.auth.models import User
from django.test import TestCase

from ishtar_common.models import PersonType, Town, IshtarSiteProfile
from archaeological_files import models
from archaeological_operations.models import Parcel, ParcelOwner
from archaeological_operations.tests import OperationInitTest


class FileInit(object):
    def login_as_superuser(self):
        self.client.login(username='username', password='tralala')

    def create_file(self):
        self.extra_models, self.model_list = {}, []
        self.user, created = User.objects.get_or_create(username='username',
                                                        is_superuser=True)
        self.user.set_password('tralala')
        self.user.save()
        self.o_user, created = User.objects.get_or_create(username='ousername')
        person_type = PersonType(label=u'Test person type',
                                 txt_idx='test_person', available=True)
        person_type.save()
        self.extra_models['person_type'] = person_type
        self.model_list.append(person_type)

        person = models.Person(surname='Surname', name='Name',
                               history_modifier=self.o_user)
        person.save()
        self.extra_models['person'] = person
        self.model_list.append(person)

        file_type = models.FileType(label=u'Test file type',
                                    txt_idx='test_file', available=True)
        file_type.save()
        self.extra_models['file_type'] = file_type
        self.model_list.append(file_type)

        dct = {'year': 2010, 'numeric_reference': 1000, 'file_type': file_type,
               'internal_reference': u'UNIT_testÉ ?', 'in_charge': person,
               'history_modifier': self.o_user, 'total_surface': 10000}
        self.item = self.model(**dct)
        self.item.save()


class FileTest(TestCase, FileInit):
    fixtures = [settings.ROOT_PATH +
                '../fixtures/initial_data-auth-fr.json',
                settings.ROOT_PATH +
                '../ishtar_common/fixtures/initial_data-fr.json']
    model = models.File

    def setUp(self):
        IshtarSiteProfile.objects.create()
        self.create_file()

    def testExternalID(self):
        self.assertEqual(
            self.item.external_id,
            u"{}-{}".format(self.item.year,
                            self.item.numeric_reference))

    def testAddAndGetHistorized(self):
        """
        Test correct new version and correct access to history
        """
        nb_hist = self.item.history.count()
        self.assertTrue(self.item.history.count() >= 1)
        base_label = self.item.internal_reference
        self.item.internal_reference = u"Unité_Test"
        self.item.history_modifier = self.user
        self.item.save()
        self.failUnlessEqual(self.item.history.count(), nb_hist + 1)
        self.failUnlessEqual(self.item.history.all()[1].internal_reference,
                             base_label)
        self.item.internal_reference = u"Unité_Testée"
        self.item.history_modifier = self.user
        self.item.skip_history_when_saving = True
        self.item.save()
        self.item.skip_history_when_saving = False
        self.failUnlessEqual(self.item.history.count(), nb_hist + 1)

    def testCreatorHistorized(self):
        """
        Test creator association
        """
        self.failUnlessEqual(self.item.history_creator, self.o_user)
        altuser, created = User.objects.get_or_create(username='altusername')
        self.item.internal_reference = u"Unité_Test"
        self.item.history_modifier = altuser
        self.item.save()
        self.failUnlessEqual(self.item.history_creator, self.o_user)

    def testIntelligentHistorisation(self):
        """
        Test that two identical version are not recorded twice in the history
        and that multiple saving in a short time are not considered
        """
        nb_hist = self.item.history.count()
        self.item.internal_reference = u"Unité_Test"
        self.item.history_modifier = self.user
        self.item.save()
        self.failUnlessEqual(self.item.history.count(), nb_hist + 1)
        nb_hist = self.item.history.count()
        self.item.save()
        self.failUnlessEqual(self.item.history.count(), nb_hist)

    def testRollbackFile(self):
        nb_hist = self.item.history.count()
        initial_values = self.item.values()
        backup_date = self.item.history.all()[0].history_date
        self.item.internal_reference = u"Unité_Test"
        self.item.history_modifier = self.user
        self.item.save()
        self.item.rollback(backup_date)
        self.failUnlessEqual(self.item.history.count(), nb_hist)
        new_values = self.item.values()
        for k in initial_values.keys():
            self.assertTrue(k in new_values)
            self.assertEqual(
                new_values[k], initial_values[k],
                msg=u"for %s: %s != %s" % (k, unicode(new_values[k]),
                                           unicode(initial_values[k])))

    def testRESTGetFile(self):
        response = self.client.post(
            '/get-file/', {'numeric_reference': self.item.numeric_reference})
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.content)
        # not allowed -> no data
        self.assertTrue(not data)

        self.login_as_superuser()
        response = self.client.post(
            '/get-file/', {'numeric_reference': self.item.numeric_reference})
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.content)
        self.assertTrue('records' in data)
        self.assertTrue(data['records'] == 1)

    def testRESTGetOldFile(self):
        initial_ref = self.item.internal_reference
        new_ref = u"Unité_Test_old_file"
        new_ref = initial_ref != new_ref and new_ref or new_ref + u"extra"
        self.item.internal_reference = new_ref
        self.item.history_modifier = self.user
        self.item.save()
        response = self.client.post(
            '/get-file/',
            {'numeric_reference': self.item.numeric_reference, 'old': 1})
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.content)
        # not allowed -> no data
        self.assertTrue(not data)

        self.login_as_superuser()
        response = self.client.post(
            '/get-file/',
            {'numeric_reference': self.item.numeric_reference, 'old': 1})
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.content)
        self.assertTrue('records' in data)
        self.assertTrue(data['records'] == 1)
        self.assertEqual(data['rows'][0]['internal_reference'], initial_ref)

# class ImporterTest(TestCase):
#    def testFormaters(self):
#        from archaeological_files import data_importer
#        for formater in [data_importer.SurfaceFormater]:
#            formater().test()


class FileOperationTest(TestCase, OperationInitTest, FileInit):
    model = models.File
    fixtures = [settings.ROOT_PATH +
                '../fixtures/initial_data-auth-fr.json',
                settings.ROOT_PATH +
                '../ishtar_common/fixtures/initial_data-fr.json',
                settings.ROOT_PATH +
                '../ishtar_common/fixtures/test_towns.json',
                settings.ROOT_PATH +
                '../ishtar_common/fixtures/initial_importtypes-fr.json',
                settings.ROOT_PATH +
                '../archaeological_files/fixtures/initial_data.json',
                settings.ROOT_PATH +
                '../archaeological_operations/fixtures/initial_data-fr.json']

    def setUp(self):
        self.create_file()
        self.orgas = self.create_orgas(self.user)
        self.operations = self.create_operation(self.user, self.orgas[0])
        self.operation = self.operations[0]

    def testFileAssociation(self):
        # parcel association
        default_town = Town.objects.all()[0]
        for p in range(0, 10):
            parcel = Parcel.objects.create(
                parcel_number=unicode(p),
                section='YY',
                town=default_town,
                operation=self.operation)
            if p == 1:
                ParcelOwner.objects.create(
                    owner=self.extra_models['person'],
                    parcel=parcel, start_date=datetime.date.today(),
                    end_date=datetime.date.today())
        initial_nb = self.item.parcels.count()
        # no parcel on the file -> new parcels are copied from the
        # operation
        self.operation.associated_file = self.item
        self.operation.save()
        self.assertEqual(self.item.parcels.count(), initial_nb + 10)
        # parcel owner well attached
        q = ParcelOwner.objects.filter(parcel__associated_file=self.item)
        self.assertEqual(q.count(), 1)

        # when attaching parcel from a file to an operation, copy is done
        parcel = Parcel.objects.create(
            parcel_number='42', section='ZZ',
            town=default_town, associated_file=self.item)
        ParcelOwner.objects.create(
            owner=self.extra_models['person'],
            parcel=parcel, start_date=datetime.date.today(),
            end_date=datetime.date.today())
        parcel.operation = self.operation
        parcel.save()
        # double reference to operation and associated_file is deleted
        self.assertEqual(parcel.operation, None)
        # now 2 objects with the same parameters
        q = Parcel.objects.filter(parcel_number='42', section='ZZ',
                                  town=default_town)
        self.assertEqual(q.count(), 2)
        q = q.filter(operation=self.operation, associated_file=None)
        self.assertEqual(q.count(), 1)
        # parcel owner well attached
        q = ParcelOwner.objects.filter(parcel__operation=self.operation,
                                       parcel__parcel_number='42')
        self.assertEqual(q.count(), 1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf import settings
from django.contrib.gis.db import models
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.db.models.signals import post_delete, post_save
from django.utils.translation import ugettext_lazy as _, ugettext, pgettext

from ishtar_common.utils import cached_label_changed

from ishtar_common.models import GeneralType, BaseHistorizedItem, \
    HistoricalRecords, OwnPerms, ShortMenuItem, Source, GeneralRelationType,\
    GeneralRecordRelations, post_delete_record_relation, get_external_id, \
    ImageModel, post_save_cache
from archaeological_operations.models import Operation, Period, Parcel


class DatingType(GeneralType):
    class Meta:
        verbose_name = _(u"Dating type")
        verbose_name_plural = _(u"Dating types")
        ordering = ('label',)
post_save.connect(post_save_cache, sender=DatingType)
post_delete.connect(post_save_cache, sender=DatingType)


class DatingQuality(GeneralType):
    class Meta:
        verbose_name = _(u"Dating quality")
        verbose_name_plural = _(u"Dating qualities")
        ordering = ('label',)
post_save.connect(post_save_cache, sender=DatingQuality)
post_delete.connect(post_save_cache, sender=DatingQuality)


class Dating(models.Model):
    period = models.ForeignKey(Period, verbose_name=_(u"Period"))
    start_date = models.IntegerField(_(u"Start date"), blank=True, null=True)
    end_date = models.IntegerField(_(u"End date"), blank=True, null=True)
    dating_type = models.ForeignKey(DatingType, verbose_name=_(u"Dating type"),
                                    blank=True, null=True)
    quality = models.ForeignKey(DatingQuality, verbose_name=_(u"Quality"),
                                blank=True, null=True)
    precise_dating = models.TextField(_(u"Precise dating"), blank=True,
                                      null=True)

    class Meta:
        verbose_name = _(u"Dating")
        verbose_name_plural = _(u"Datings")

    def __unicode__(self):
        start_date = self.start_date and unicode(self.start_date) or u""
        end_date = self.end_date and unicode(self.end_date) or u""
        if not start_date and not end_date:
            return unicode(self.period)
        return u"%s (%s-%s)" % (self.period, start_date, end_date)


class Unit(GeneralType):
    order = models.IntegerField(_(u"Order"))
    parent = models.ForeignKey("Unit", verbose_name=_(u"Parent unit"),
                               blank=True, null=True)

    class Meta:
        verbose_name = _(u"Unit Type")
        verbose_name_plural = _(u"Unit Types")
        ordering = ('order',)

    def __unicode__(self):
        return self.label
post_save.connect(post_save_cache, sender=Unit)
post_delete.connect(post_save_cache, sender=Unit)


class ActivityType(GeneralType):
    order = models.IntegerField(_(u"Order"))

    class Meta:
        verbose_name = _(u"Activity Type")
        verbose_name_plural = _(u"Activity Types")
        ordering = ('order',)

    def __unicode__(self):
        return self.label
post_save.connect(post_save_cache, sender=ActivityType)
post_delete.connect(post_save_cache, sender=ActivityType)


class IdentificationType(GeneralType):
    order = models.IntegerField(_(u"Order"))

    class Meta:
        verbose_name = _(u"Identification Type")
        verbose_name_plural = _(u"Identification Types")
        ordering = ('order', 'label')

    def __unicode__(self):
        return self.label
post_save.connect(post_save_cache, sender=IdentificationType)
post_delete.connect(post_save_cache, sender=IdentificationType)


class ContextRecord(BaseHistorizedItem, ImageModel, OwnPerms, ShortMenuItem):
    SHOW_URL = 'show-contextrecord'
    SLUG = 'contextrecord'
    TABLE_COLS = ['parcel.town', 'operation.year',
                  'operation.operation_code',
                  'label', 'unit']
    if settings.COUNTRY == 'fr':
        TABLE_COLS.insert(1, 'operation.code_patriarche')
    TABLE_COLS_FOR_OPE = ['label', 'parcel', 'unit',
                          'datings.period', 'description']
    TABLE_COLS_FOR_OPE_LBL = {'section__parcel_number': _("Parcel")}
    CONTEXTUAL_TABLE_COLS = {
        'full': {
            'related_context_records': 'detailled_related_context_records'
        }
    }
    IMAGE_PREFIX = 'context_records/'
    external_id = models.TextField(_(u"External ID"), blank=True, null=True)
    auto_external_id = models.BooleanField(
        _(u"External ID is set automatically"), default=False)
    parcel = models.ForeignKey(Parcel, verbose_name=_(u"Parcel"),
                               related_name='context_record')
    operation = models.ForeignKey(Operation, verbose_name=_(u"Operation"),
                                  related_name='context_record')
    label = models.CharField(_(u"ID"), max_length=200)
    description = models.TextField(_(u"Description"), blank=True, null=True)
    comment = models.TextField(_(u"General comment"), blank=True, null=True)
    opening_date = models.DateField(_(u"Date d'ouverture"),
                                    blank=True, null=True)
    closing_date = models.DateField(_(u"End date"), blank=True, null=True)
    length = models.FloatField(_(u"Length (m)"), blank=True, null=True)
    width = models.FloatField(_(u"Width (m)"), blank=True, null=True)
    thickness = models.FloatField(_(u"Thickness (m)"), blank=True,
                                  null=True)
    depth = models.FloatField(_(u"Depth (m)"), blank=True, null=True)
    location = models.CharField(
        _(u"Location"), blank=True, null=True, max_length=200,
        help_text=_(u"A short description of the location of the context "
                    u"record"))
    datings = models.ManyToManyField(Dating)
    datings_comment = models.TextField(_(u"Comment on datings"), blank=True,
                                       null=True)
    unit = models.ForeignKey(Unit, verbose_name=_(u"Unit"), related_name='+',
                             blank=True, null=True)
    has_furniture = models.NullBooleanField(_(u"Has furniture?"), blank=True,
                                            null=True)
    filling = models.TextField(_(u"Filling"), blank=True, null=True)
    interpretation = models.TextField(_(u"Interpretation"), blank=True,
                                      null=True)
    taq = models.IntegerField(
        _(u"TAQ"), blank=True, null=True,
        help_text=_(u"\"Terminus Ante Quem\" the context record can't have "
                    u"been created after this date"))
    taq_estimated = models.IntegerField(
        _(u"Estimated TAQ"), blank=True, null=True,
        help_text=_(u"Estimation of a \"Terminus Ante Quem\""))
    tpq = models.IntegerField(
        _(u"TPQ"), blank=True, null=True,
        help_text=_(u"\"Terminus Post Quem\" the context record can't have "
                    u"been created before this date"))
    tpq_estimated = models.IntegerField(
        _(u"Estimated TPQ"), blank=True, null=True,
        help_text=_(u"Estimation of a \"Terminus Post Quem\""))
    identification = models.ForeignKey(
        IdentificationType, blank=True, null=True,
        verbose_name=_(u"Identification"),)
    activity = models.ForeignKey(ActivityType, blank=True, null=True,
                                 verbose_name=_(u"Activity"),)
    related_context_records = models.ManyToManyField(
        'ContextRecord', through='RecordRelations', blank=True, null=True)
    point = models.PointField(_(u"Point"), blank=True, null=True, dim=3)
    polygon = models.PolygonField(_(u"Polygon"), blank=True, null=True)
    cached_label = models.TextField(_(u"Cached name"), null=True, blank=True)
    history = HistoricalRecords()

    class Meta:
        verbose_name = _(u"Context Record")
        verbose_name_plural = _(u"Context Record")
        permissions = (
            ("view_contextrecord", ugettext(u"Can view all Context Records")),
            ("view_own_contextrecord",
             ugettext(u"Can view own Context Record")),
            ("add_own_contextrecord",
             ugettext(u"Can add own Context Record")),
            ("change_own_contextrecord",
             ugettext(u"Can change own Context Record")),
            ("delete_own_contextrecord",
             ugettext(u"Can delete own Context Record")),
        )
        ordering = ('cached_label',)

    @property
    def name(self):
        return self.label or ""

    @property
    def short_class_name(self):
        return pgettext("short", u"Context record")

    def __unicode__(self):
        return self.short_label

    @property
    def short_label(self):
        return settings.JOINT.join([unicode(item) for item in [
            self.operation.get_reference(), self.parcel, self.label] if item])

    @property
    def show_url(self):
        return reverse('show-contextrecord', args=[self.pk, ''])

    @classmethod
    def get_query_owns(cls, user):
        return Q(operation__scientist=user.ishtaruser.person) |\
            Q(operation__in_charge=user.ishtaruser.person) |\
            Q(history_creator=user)

    @classmethod
    def get_owns(cls, user, menu_filtr=None, limit=None):
        replace_query = {}
        if menu_filtr:
            replace_query = {'operation': menu_filtr}
        owns = super(ContextRecord, cls).get_owns(
            user, replace_query=replace_query,
            limit=limit)
        return sorted(owns, key=lambda x: x.cached_label)

    def full_label(self):
        return unicode(self)

    def _real_label(self):
        if not self.operation.code_patriarche:
            return
        return settings.JOINT.join((unicode(self.operation.code_patriarche),
                                    self.label))

    def _temp_label(self):
        if self.operation.code_patriarche:
            return
        return settings.JOINT.join([unicode(lbl) for lbl in [
            self.operation.year, self.operation.operation_code, self.label]
            if lbl])

    def _generate_cached_label(self):
        return self.full_label()

    @property
    def reference(self):
        if not self.operation:
            return "00"
        return self.full_label()

    def get_department(self):
        if not self.operation:
            return "00"
        return self.operation.get_department()

    def get_town_label(self):
        if not self.operation:
            return "00"
        return self.operation.get_town_label()

    @classmethod
    def get_periods(cls, slice='year', fltr={}):
        q = cls.objects
        if fltr:
            q = q.filter(**fltr)
        if slice == 'year':
            years = set()
            for res in list(q.values('operation__start_date')):
                if res['operation__start_date']:
                    yr = res['operation__start_date'].year
                    years.add(yr)
            return list(years)
        return []

    @classmethod
    def get_by_year(cls, year, fltr={}):
        q = cls.objects
        if fltr:
            q = q.filter(**fltr)
        return q.filter(operation__start_date__year=year)

    @classmethod
    def get_operations(cls):
        return [dct['operation__pk']
                for dct in cls.objects.values('operation__pk').distinct()]

    @classmethod
    def get_by_operation(cls, operation_id):
        return cls.objects.filter(operation__pk=operation_id)

    @classmethod
    def get_total_number(cls, fltr={}):
        q = cls.objects
        if fltr:
            q = q.filter(**fltr)
        return q.count()

    def detailled_related_context_records(self):
        crs = []
        for cr in self.right_relations.all():
            crs.append(u"{} ({})".format(cr.right_record,
                                         cr.relation_type.get_tiny_label()))
        return u" ; ".join(crs)

    def find_docs_q(self):
        from archaeological_finds.models import FindSource
        return FindSource.objects.filter(find__base_finds__context_record=self)

    def save(self, *args, **kwargs):
        returned = super(ContextRecord, self).save(*args, **kwargs)
        updated = False
        if not self.external_id or self.auto_external_id:
            external_id = get_external_id('context_record_external_id', self)
            if external_id != self.external_id:
                updated = True
                self.auto_external_id = True
                self.external_id = external_id
        if updated:
            self.save()
        return returned


post_save.connect(cached_label_changed, sender=ContextRecord)


class RelationType(GeneralRelationType):
    inverse_relation = models.ForeignKey(
        'RelationType', verbose_name=_(u"Inverse relation"), blank=True,
        null=True)

    class Meta:
        verbose_name = _(u"Relation type")
        verbose_name_plural = _(u"Relation types")
        ordering = ('order', 'label')


class RecordRelations(GeneralRecordRelations, models.Model):
    MAIN_ATTR = 'left_record'
    left_record = models.ForeignKey(ContextRecord,
                                    related_name='right_relations')
    right_record = models.ForeignKey(ContextRecord,
                                     related_name='left_relations')
    relation_type = models.ForeignKey(RelationType)
    TABLE_COLS = [
        "left_record__label", "left_record__unit", "left_record__parcel",
        "left_record__datings__period", "left_record__description",
        "relation_type",
        "right_record__label", "right_record__unit", "right_record__parcel",
        "right_record__datings__period", "right_record__description"]
    TABLE_COLS_LBL = {
        "left_record__label": _(u"ID (left)"),
        "left_record__unit": _(u"Unit (left)"),
        "left_record__parcel": _(u"Parcel (left)"),
        "left_record__description": _(u"Description (left)"),
        "left_record__datings__period": _(u"Periods (left)"),
        "relation_type": _(u"Relation type"),
        "right_record__label": _(u"Unit (right)"),
        "right_record__unit": _(u"Parcel (right)"),
        "right_record__parcel": _(u"Parcel (right)"),
        "right_record__description": _(u"Description (right)"),
        "right_record__datings__period": _(u"Periods (right)")
    }

    class Meta:
        verbose_name = _(u"Record relation")
        verbose_name_plural = _(u"Record relations")

post_delete.connect(post_delete_record_relation, sender=RecordRelations)


class ContextRecordSource(Source):
    SHOW_URL = 'show-contextrecordsource'
    MODIFY_URL = 'record_source_modify'
    TABLE_COLS = ['context_record__operation', 'context_record'] + \
        Source.TABLE_COLS

    class Meta:
        verbose_name = _(u"Context record documentation")
        verbose_name_plural = _(u"Context record documentations")
    context_record = models.ForeignKey(
        ContextRecord, verbose_name=_(u"Context record"),
        related_name="source")

    @property
    def owner(self):
        return self.context_record



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf.urls.defaults import *

from ishtar_common.wizards import check_rights
import views

# be carreful: each check_rights must be relevant with ishtar_menu

# forms:
urlpatterns = patterns(
    '',
    url(r'file_administrativeactfil_search/(?P<step>.+)?$',
        check_rights(['change_administrativeact'])(
            views.file_administrativeactfile_search_wizard),
        name='file_administrativeactfile_search'),
    url(r'file_administrativeactfil/(?P<step>.+)?$',
        check_rights(['change_administrativeact'])(
            views.file_administrativeactfile_wizard),
        name='file_administrativeactfile'),
    url(r'file_administrativeactfile_modify/(?P<pk>.+)/$',
        views.file_administrativeactfile_modify,
        name='file_administrativeactfile_modify'),
    url(r'file_administrativeactfil_deletion/(?P<step>.+)?$',
        check_rights(['change_administrativeact'])(
            views.file_administrativeactfile_deletion_wizard),
        name='file_administrativeactfile_deletion'),
    url(r'file_administrativeactfil_modification/(?P<step>.+)?$',
        check_rights(['change_administrativeact'])(
            views.file_administrativeactfile_modification_wizard),
        name='file_administrativeactfile_modification'),
    url(r'file_search/(?P<step>.+)?$',
        check_rights(['view_file', 'view_own_file'])(
            views.file_search_wizard),
        name='file_search'),
    url(r'file_creation/(?P<step>.+)?$',
        check_rights(['add_file'])(
            views.file_creation_wizard), name='file_creation'),
    url(r'file_modification/(?P<step>.+)?$',
        check_rights(['change_file', 'change_own_file'])(
            views.file_modification_wizard), name='file_modification'),
    url(r'file_modify/(?P<pk>.+)/$', views.file_modify, name='file_modify'),
    url(r'file_closing/(?P<step>.+)?$',
        check_rights(['change_file'])(
            views.file_closing_wizard),
        name='file_closing'),
    url(r'file_deletion/(?P<step>.+)?$',
        check_rights(['delete_file', 'delete_own_file'])(
            views.file_deletion_wizard),
        name='file_deletion'),
)

urlpatterns += patterns(
    'archaeological_files.views',
    url(r'autocomplete-file/$', 'autocomplete_file',
        name='autocomplete-file'),
    url(r'get-file/(?P<type>.+)?$', 'get_file',
        name='get-file'),
    url(r'get-file-full/(?P<type>.+)?$', 'get_file',
        name='get-file-full', kwargs={'full': True}),
    url(r'get-file-shortcut/(?P<type>.+)?$',
        'get_file', name='get-file-shortcut',
        kwargs={'full': 'shortcut'}),
    url(r'get-administrativeactfile/(?P<type>.+)?$',
        'get_administrativeactfile', name='get-administrativeactfile'),
    url(r'show-file(?:/(?P<pk>.+))?/(?P<type>.+)?$', 'show_file',
        name='show-file'),
    url(r'show-historized-file/(?P<pk>.+)?/(?P<date>.+)?$',
        'show_file', name='show-historized-file'),
    url(r'revert-file/(?P<pk>.+)/(?P<date>.+)$',
        'revert_file', name='revert-file'),
    url(r'dashboard_file/$', 'dashboard_file', name='dashboard-file'),
)

urlpatterns += patterns(
    'archaeological_operations.views',
    url(r'file_administrativeact_document/$',
        'administrativeactfile_document',
        name='file-administrativeact-document', kwargs={'operation': False}),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf import settings
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase
from ishtar_common.models import ImporterType, IshtarUser, ImporterColumn,\
    FormaterType, ImportTarget

from archaeological_finds import models

from archaeological_context_records.tests import ImportContextRecordTest, \
    ContextRecordInit

from ishtar_common import forms_common


class ImportFindTest(ImportContextRecordTest):
    test_context_records = False

    fixtures = ImportContextRecordTest.fixtures + [
        settings.ROOT_PATH +
        '../archaeological_finds/fixtures/initial_data-fr.json',
    ]

    def testMCCImportFinds(self, test=True):
        self.testMCCImportContextRecords(test=False)

        old_nb = models.BaseFind.objects.count()
        old_nb_find = models.Find.objects.count()
        MCC = ImporterType.objects.get(name=u"MCC - Mobilier")

        col = ImporterColumn.objects.create(col_number=25,
                                            importer_type_id=MCC.pk)
        formater = FormaterType.objects.filter(
            formater_type='FileFormater').all()[0]
        ImportTarget.objects.create(target='find__image',
                                    formater_type_id=formater.pk,
                                    column_id=col.pk)
        mcc_file = open(
            settings.ROOT_PATH +
            '../archaeological_finds/tests/MCC-finds-example.csv', 'rb')
        mcc_images = open(
            settings.ROOT_PATH +
            '../archaeological_finds/tests/images.zip', 'rb')
        file_dict = {'imported_file': SimpleUploadedFile(mcc_file.name,
                                                         mcc_file.read()),
                     'imported_images': SimpleUploadedFile(mcc_images.name,
                                                           mcc_images.read())}
        post_dict = {'importer_type': MCC.pk, 'skip_lines': 1,
                     "encoding": 'utf-8'}
        form = forms_common.NewImportForm(data=post_dict, files=file_dict,
                                          instance=None)
        form.is_valid()
        if test:
            self.assertTrue(form.is_valid())
        impt = form.save(self.ishtar_user)
        impt.initialize()

        # doing manual connections
        ceram = models.MaterialType.objects.get(txt_idx='ceramic').pk
        glass = models.MaterialType.objects.get(txt_idx='glass').pk
        self.setTargetKey('find__material_types', 'terre-cuite', ceram)
        self.setTargetKey('find__material_types', 'verre', glass)
        impt.importation()
        if not test:
            return
        # new finds has now been imported
        current_nb = models.BaseFind.objects.count()
        self.assertEqual(current_nb, (old_nb + 4))
        current_nb = models.Find.objects.count()
        self.assertEqual(current_nb, (old_nb_find + 4))
        self.assertEqual(
            models.Find.objects.filter(material_types__pk=ceram).count(), 4)
        self.assertEqual(
            models.Find.objects.filter(material_types__pk=glass).count(), 1)
        images = [f.image for f in models.Find.objects.all() if f.image.name]
        self.assertEqual(len(images), 1)


class FindInit(ContextRecordInit):
    test_context_records = False

    def create_finds(self, user=None, data_base={}, data={}, force=False):
        if not getattr(self, 'finds', None):
            self.finds = []
        if not getattr(self, 'base_finds', None):
            self.base_finds = []

        default = {'label': "Base find"}
        if not data_base.get('history_modifier'):
            data_base['history_modifier'] = self.get_default_user()
        if force or not data_base.get('context_record'):
            data_base['context_record'] = self.get_default_context_record(
                force=force)
        default.update(data_base)
        base_find = models.BaseFind.objects.create(**default)
        self.base_finds.append(base_find)

        data["history_modifier"] = data_base["history_modifier"]
        find = models.Find.objects.create(**data)
        find.base_finds.add(base_find)
        self.finds.append(find)
        return self.finds, self.base_finds

    def get_default_find(self):
        return self.create_finds()[0]

    def tearDown(self):
        super(FindInit, self).tearDown()
        if hasattr(self, 'finds'):
            for f in self.finds:
                try:
                    f.delete()
                except:
                    pass
            self.finds = []
        if hasattr(self, 'base_finds'):
            for f in self.base_finds:
                try:
                    f.delete()
                except:
                    pass
            self.base_find = []


class FindTest(FindInit, TestCase):
    fixtures = [settings.ROOT_PATH +
                '../fixtures/initial_data.json',
                settings.ROOT_PATH +
                '../ishtar_common/fixtures/initial_data.json',
                settings.ROOT_PATH +
                '../archaeological_files/fixtures/initial_data.json',
                settings.ROOT_PATH +
                '../archaeological_operations/fixtures/initial_data-fr.json',
                settings.ROOT_PATH +
                '../archaeological_finds/fixtures/initial_data-fr.json',
                ]
    model = models.Find

    def setUp(self):
        self.create_finds(force=True)

    def testExternalID(self):
        find = self.finds[0]
        base_find = find.base_finds.all()[0]
        self.assertEqual(
            find.external_id,
            u"{}-{}".format(
                find.get_first_base_find().context_record.external_id,
                find.label))
        self.assertEqual(
            base_find.external_id,
            u"{}-{}".format(
                base_find.context_record.external_id,
                base_find.label))


class PackagingTest(FindInit, TestCase):
    fixtures = [settings.ROOT_PATH +
                '../fixtures/initial_data.json',
                settings.ROOT_PATH +
                '../ishtar_common/fixtures/initial_data.json',
                settings.ROOT_PATH +
                '../archaeological_files/fixtures/initial_data.json',
                settings.ROOT_PATH +
                '../archaeological_operations/fixtures/initial_data-fr.json',
                settings.ROOT_PATH +
                '../archaeological_finds/fixtures/initial_data-fr.json',
                ]
    model = models.Find

    def setUp(self):
        self.create_finds({"label": u"Find 1"}, force=True)
        self.create_finds({"label": u"Find 2"}, force=True)
        self.basket = models.FindBasket.objects.create(
            label="My basket", user=IshtarUser.objects.get(
                pk=self.get_default_user().pk))
        self.other_basket = models.FindBasket.objects.create(
            label="My other basket", user=IshtarUser.objects.get(
                pk=self.get_default_user().pk))
        for find in self.finds:
            self.basket.items.add(find)
            self.other_basket.items.add(find)

    def testPackaging(self):
        treatment_type = models.TreatmentType.objects.get(txt_idx='packaging')
        treatment = models.Treatment(treatment_type=treatment_type)
        items_nb = models.Find.objects.count()
        treatment.save(user=self.get_default_user(), items=self.basket)
        self.assertEqual(items_nb + self.basket.items.count(),
                         models.Find.objects.count(),
                         msg="Packaging doesn't generate enough new finds")
        # new version of the find is in the basket
        for item in self.basket.items.all():
            self.assertNotIn(
                item, self.finds,
                msg="Original basket have not been upgraded after packaging")
        for item in self.other_basket.items.all():
            self.assertNotIn(
                item, self.finds,
                msg="Other basket have not been upgraded after packaging")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.utils.translation import ugettext_lazy as _

from ishtar_common.menu_base import SectionItem, MenuItem

import models

# be carreful: each access_controls must be relevant with check_rights in urls

MENU_SECTIONS = [
    (50,
     SectionItem(
         'find_management', _(u"Find"),
         profile_restriction='find',
         childs=[
             MenuItem(
                 'find_search', _(u"Search"),
                 model=models.Find,
                 access_controls=['view_find',
                                  'view_own_find']),
             MenuItem(
                 'find_creation', _(u"Creation"),
                 model=models.Find,
                 access_controls=['add_find',
                                  'add_own_find']),
             MenuItem(
                 'find_modification', _(u"Modification"),
                 model=models.Find,
                 access_controls=['change_find',
                                  'change_own_find']),
             # MenuItem(
             #     'treatment_creation', _(u"Add a treatment"),
             #     model=models.Treatment,
             #     access_controls=['change_find',
             #                      'change_own_find']),
             MenuItem(
                 'find_deletion', _(u"Deletion"),
                 model=models.Find,
                 access_controls=['change_find',
                                  'change_own_find']),
             SectionItem(
                 'find_basket', _(u"Basket"),
                 childs=[
                     MenuItem('find_basket_creation',
                              _(u"Creation"),
                              model=models.FindBasket,
                              access_controls=['change_find',
                                               'change_own_find']),
                     MenuItem('find_basket_modification_add',
                              _(u"Manage items"),
                              model=models.FindBasket,
                              access_controls=[
                                  'change_find',
                                  'change_own_find']),
                     MenuItem('find_basket_deletion',
                              _(u"Deletion"),
                              model=models.FindBasket,
                              access_controls=['change_find',
                                               'change_own_find']),
                 ]),
             SectionItem(
                 'find_source', _(u"Documentation"),
                 childs=[
                     MenuItem('find_source_search',
                              _(u"Search"),
                              model=models.FindSource,
                              access_controls=['view_find',
                                               'view_own_find']),
                     MenuItem('find_source_creation',
                              _(u"Creation"),
                              model=models.FindSource,
                              access_controls=['change_find',
                                               'change_own_find']),
                     MenuItem('find_source_modification',
                              _(u"Modification"),
                              model=models.FindSource,
                              access_controls=['change_find',
                                               'change_own_find']),
                     MenuItem('find_source_deletion',
                              _(u"Deletion"),
                              model=models.FindSource,
                              access_controls=['change_find',
                                               'change_own_find']),
                 ])
         ]))
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>
# Copyright (C) 2007  skam <massimo dot scamarcia at gmail.com>
#                          (http://djangosnippets.org/snippets/233/)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django import forms
from django.conf import settings
from django.core.urlresolvers import reverse
from django.db.models import fields
from django.forms import ClearableFileInput
from django.forms.widgets import flatatt
from django.template import Context, loader
from django.template.defaultfilters import slugify
from django.utils.encoding import smart_unicode
from django.utils.functional import lazy
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.utils.simplejson import JSONEncoder
from django.utils.translation import ugettext_lazy as _

from ishtar_common import models

reverse_lazy = lazy(reverse, unicode)


class Select2Multiple(forms.SelectMultiple):
    class Media:
        css = {
            'all': ('select2/css/select2.css',)
        }
        js = ('select2/js/select2.min.js',
              'select2/js/init.js')

    def render(self, name, value, attrs=None, choices=()):
        klass = attrs and attrs.get('class') or ''
        klass += ' ' if klass else '' + 'js-select2'
        if not attrs:
            attrs = {}
        attrs['class'] = klass
        return super(Select2Multiple, self).render(name, value, attrs,
                                                   choices)


class MultipleAutocompleteField(forms.MultipleChoiceField):
    def __init__(self, *args, **kwargs):
        self.model = None
        if 'model' in kwargs:
            self.model = kwargs.pop('model')
        if 'choices' not in kwargs and self.model:
            kwargs['choices'] = []
        new = kwargs.pop('new') if 'new' in kwargs else None
        if 'widget' not in kwargs and self.model:
            kwargs['widget'] = JQueryAutoComplete(
                reverse_lazy('autocomplete-' + self.model.__name__.lower()),
                associated_model=self.model, new=new,
                multiple=True)
        super(MultipleAutocompleteField, self).__init__(*args, **kwargs)

    def get_choices(self):
        return [(i.pk, unicode(i)) for i in self.model.objects.all()]

    def valid_value(self, value):
        if not self.model:
            return super(MultipleAutocompleteField, self).valid_value(value)
        return bool(self.model.objects.filter(pk=value).count())

    def clean(self, value):
        if value:
            # clean JS messup with values
            try:
                if type(value) not in (list, tuple):
                    value = [int(value)]
                else:
                    val = value
                    value = []
                    for v in val:
                        v = unicode(v).strip('[').strip(']')\
                                      .strip('u').strip("'").strip('"')
                        value += [int(va.strip())
                                  for va in list(set(v.split(',')))
                                  if va.strip()]
            except (TypeError, ValueError):
                value = []
        else:
            value = []
        return super(MultipleAutocompleteField, self).clean(value)


class DeleteWidget(forms.CheckboxInput):
    def render(self, name, value, attrs=None):
        final_attrs = flatatt(self.build_attrs(attrs, name=name,
                                               value='1'))
        output = ['<tr class="delete"><td colspan="2">']
        output.append(u"<button%s>%s</button>" % (final_attrs, _("Delete")))
        output.append('</td></tr>')
        return mark_safe('\n'.join(output))


class ImageFileInput(ClearableFileInput):
    template_with_initial = u'<span class="prettyPhoto">%(initial)s</span>'\
        u' %(clear_template)s<br />%(input_text)s: %(input)s'


class SquareMeterWidget(forms.TextInput):
    def render(self, name, value, attrs=None):
        if not value:
            value = u""
        final_attrs = flatatt(self.build_attrs(attrs, name=name, value=value))
        dct = {'final_attrs': final_attrs,
               'unit': settings.SURFACE_UNIT_LABEL,
               'id': attrs['id'],
               "safe_id": attrs['id'].replace('-', '_')}
        t = loader.get_template('blocks/SquareMeterWidget.html')
        rendered = t.render(Context(dct))
        return mark_safe(rendered)

AreaWidget = forms.TextInput

if settings.SURFACE_UNIT == 'square-metre':
    AreaWidget = SquareMeterWidget


class JQueryDate(forms.TextInput):
    def __init__(self, *args, **kwargs):
        super(JQueryDate, self).__init__(*args, **kwargs)
        if 'class' not in self.attrs:
            self.attrs['class'] = ''
        self.attrs['class'] = 'date-pickup'

    def render(self, name, value=None, attrs=None):
        if value:
            value = unicode(value)
        # very specific...
        if settings.COUNTRY == 'fr' and value and '/' in value:
            values = value.split('/')
            if len(values) == 3:
                value = "%s-%s-%s" % (values[2], values[1], values[0])
        if not attrs:
            attrs = {}
        attrs['autocomplete'] = 'off'
        rendered = super(JQueryDate, self).render(name, value, attrs)
        # use window.onload to be sure that datepicker don't interfere
        # with autocomplete fields
        var_name = name.replace('-', '_')
        rendered += """
<script type="text/javascript"><!--//
    function load_jquerydate_%(var_name)s(){
        $(".date-pickup").datepicker($.datepicker.regional["%(country)s"]);
        var val = $("#id_%(name)s").val();
        if(val){
            var dt = $.datepicker.parseDate('yy-mm-dd', val);
            val = $.datepicker.formatDate(
                    $.datepicker.regional["%(country)s"]['dateFormat'],
                    dt);
            $("#id_%(name)s").val(val);
        }
    }
    $(window).load(load_jquerydate_%(var_name)s);
//--></script>
""" % {"name": name, "var_name": var_name, "country": settings.COUNTRY}
        return rendered


class JQueryAutoComplete(forms.TextInput):
    def __init__(self, source, associated_model=None, options={}, attrs={},
                 new=False, url_new='', multiple=False, limit={},
                 dynamic_limit=[]):
        """
        Source can be a list containing the autocomplete values or a
        string containing the url used for the request.
        """
        self.options = None
        self.attrs = {}
        self.source = source
        self.associated_model = associated_model
        if len(options) > 0:
            self.options = JSONEncoder().encode(options)
        self.attrs.update(attrs)
        self.new = new
        self.url_new = url_new
        self.multiple = multiple
        self.limit = limit
        self.dynamic_limit = dynamic_limit

    def value_from_datadict(self, data, files, name):
        if self.multiple:
            return data.getlist(name, None)
        else:
            return data.get(name, None)

    def render_js(self, field_id):
        if isinstance(self.source, list):
            source = JSONEncoder().encode(self.source)
        elif isinstance(self.source, str) or isinstance(self.source, unicode):
            source = "'%s'" % escape(self.source)
        else:
            try:
                source = "'" + unicode(self.source) + "'"
            except:
                raise ValueError('source type is not valid')
        dynamic_limit = [
            'id_' + lim.replace('_', '') + '-' +
            '-'.join(field_id.split('-')[1:-1]) + '-' + lim
            for lim in self.dynamic_limit
        ]
        dct = {'source': mark_safe(source),
               'field_id': field_id,
               'dynamic_limit': dynamic_limit}
        if self.options:
            dct['options'] = mark_safe('%s' % self.options)

        js = ""
        tpl = 'blocks/JQueryAutocomplete.js'
        if self.multiple:
            tpl = 'blocks/JQueryAutocompleteMultiple.js'
        t = loader.get_template(tpl)
        js = t.render(Context(dct))
        return js

    def render(self, name, value=None, attrs=None):
        attrs_hidden = self.build_attrs(attrs, name=name)
        attrs_select = self.build_attrs(attrs)
        attrs_select['placeholder'] = _(u"Search...")
        if value:
            hiddens = []
            selects = []
            values = value
            if type(value) not in (list, tuple):
                values = unicode(escape(smart_unicode(value)))
                values = values.replace('[', '').replace(']', '')
                values = values.split(',')
            else:
                values = []
                for v in value:
                    values += v.split(',')
            for v in values:
                if not v:
                    continue
                hiddens.append(v)
                selects.append(v)
                if self.associated_model:
                    try:
                        selects[-1] = unicode(
                            self.associated_model.objects.get(pk=v))
                    except (self.associated_model.DoesNotExist, ValueError):
                        selects.pop()
                        hiddens.pop()
            if self.multiple:
                attrs_hidden['value'] = ", ".join(hiddens)
                if selects:
                    selects.append("")
                attrs_select['value'] = ", ".join(selects)
            else:
                if hiddens and selects:
                    attrs_hidden['value'] = hiddens[0]
                    attrs_select['value'] = selects[0]
        if 'id' not in self.attrs:
            attrs_hidden['id'] = 'id_%s' % name
            attrs_select['id'] = 'id_select_%s' % name
        if 'class' not in attrs_select:
            attrs_select['class'] = 'autocomplete'
        new = ''
        if self.new:
            model_name = self.associated_model._meta.object_name.lower()
            limits = []
            for k in self.limit:
                limits.append(k + "__" + "-".join(
                              [unicode(v) for v in self.limit[k]]))
            args = [attrs_select['id']]
            if limits:
                args.append(';'.join(limits))
            url_new = 'new-' + model_name
            if self.url_new:
                url_new = self.url_new
            url_new = reverse(url_new, args=args)
            new = u'  <a href="#" class="add-button" '\
                  u'onclick="open_window(\'%s\');">+</a>' % url_new
        html = u'''<input%(attrs_select)s/>%(new)s\
<input type="hidden"%(attrs_hidden)s/>\
        <script type="text/javascript"><!--//
        %(js)s//--></script>
        ''' % {
            'attrs_select': flatatt(attrs_select),
            'attrs_hidden': flatatt(attrs_hidden),
            'js': self.render_js(name),
            'new': new
        }
        return html


class JQueryTown(forms.TextInput):
    """
    Town fields whith state and department pre-selections
    """

    def __init__(self, source, options={},
                 attrs={}, new=False, limit={}):
        self.options = None
        self.attrs = {}
        self.source = source
        if len(options) > 0:
            self.options = JSONEncoder().encode(options)
        self.attrs.update(attrs)
        self.new = new
        self.limit = limit

    @classmethod
    def encode_source(cls, source):
        encoded_src = ''
        if isinstance(source, list):
            encoded_src = JSONEncoder().encode(source)
        elif isinstance(source, str) \
                or isinstance(source, unicode):
            src = escape(source)
            if not src.endswith('/'):
                src += "/"
            encoded_src = "'%s'" % src
        else:
            try:
                src = unicode(source)
                if not src.endswith('/'):
                    src += "/"
                encoded_src = "'%s'" % src
            except:
                raise ValueError('source type is not valid')
        return encoded_src

    def render(self, name, value=None, attrs=None):
        attrs_hidden = self.build_attrs(attrs, name=name)
        attrs_select = self.build_attrs(attrs)
        attrs_select['placeholder'] = _(u"Search...")
        selected = ''
        selected_state = ''
        selected_department = ''
        if value:
            hiddens = []
            selects = []
            if type(value) not in (list, tuple):
                values = unicode(escape(smart_unicode(value)))
                values = values.replace('[', '').replace(']', '')
                values = values.split(',')
            else:
                values = []
                for v in value:
                    values += v.split(',')
            for v in values:
                if not v:
                    continue
                hiddens.append(v)
                selects.append(v)
                try:
                    item = models.Town.objects.get(pk=v)
                    selects[-1] = unicode(item)
                    if item.departement:
                        selected_department = item.departement.number
                        if item.departement.state:
                            selected_state = item.departement.state.number
                    selected = item.pk
                except (models.Town.DoesNotExist, ValueError):
                    selects.pop()
                    hiddens.pop()
            if hiddens and selects:
                attrs_hidden['value'] = hiddens[0]
                attrs_select['value'] = selects[0]
        if 'id' not in self.attrs:
            attrs_hidden['id'] = 'id_%s' % name
            attrs_select['id'] = 'id_select_%s' % name
        if 'class' not in attrs_select:
            attrs_select['class'] = 'autocomplete'

        source = self.encode_source(self.source)
        dct = {'source': mark_safe(source),
               'selected': selected,
               'safe_field_id': slugify(name).replace('-', '_'),
               'field_id': name}
        if self.options:
            dct['options'] = mark_safe('%s' % self.options)

        dct.update({'attrs_select': mark_safe(flatatt(attrs_select)),
                    'attrs_hidden': mark_safe(flatatt(attrs_hidden)),
                    'name': name,
                    'states': models.State.objects.all().order_by('label'),
                    'selected_department': selected_department,
                    'selected_state': selected_state}
                   )
        html = loader.get_template('blocks/JQueryAdvancedTown.html')\
                     .render(Context(dct))
        return html


class JQueryPersonOrganization(forms.TextInput):
    """
    Complex widget which manage:
     * link between person and organization
     * display addresses of the person and of the organization
     * create new person and new organization
    """

    def __init__(self, source, edit_source, model, options={},
                 attrs={}, new=False, limit={},
                 html_template='blocks/PersonOrganization.html',
                 js_template='blocks/JQueryPersonOrganization.js'):
        self.options = None
        self.attrs = {}
        self.model = model
        self.source = source
        self.edit_source = edit_source
        if len(options) > 0:
            self.options = JSONEncoder().encode(options)
        self.attrs.update(attrs)
        self.new = new
        self.limit = limit
        self.js_template = js_template
        self.html_template = html_template

    @classmethod
    def encode_source(cls, source):
        encoded_src = ''
        if isinstance(source, list):
            encoded_src = JSONEncoder().encode(source)
        elif isinstance(source, str) \
                or isinstance(source, unicode):
            encoded_src = "'%s'" % escape(source)
        else:
            try:
                encoded_src = "'" + unicode(source) + "'"
            except:
                raise ValueError('source type is not valid')
        return encoded_src

    def render_js(self, field_id, selected=''):
        source = self.encode_source(self.source)
        edit_source = self.encode_source(self.edit_source)
        dct = {'source': mark_safe(source),
               'edit_source': mark_safe(edit_source),
               'selected': selected,
               'safe_field_id': slugify(field_id).replace('-', '_'),
               'field_id': field_id}
        if self.options:
            dct['options'] = mark_safe('%s' % self.options)
        js = loader.get_template(self.js_template).render(Context(dct))
        return js

    def render(self, name, value=None, attrs=None):
        attrs_hidden = self.build_attrs(attrs, name=name)
        attrs_select = self.build_attrs(attrs)
        attrs_select['placeholder'] = _(u"Search...")
        selected = ''
        if value:
            hiddens = []
            selects = []
            if type(value) not in (list, tuple):
                values = unicode(escape(smart_unicode(value)))
                values = values.replace('[', '').replace(']', '')
                values = values.split(',')
            else:
                values = []
                for v in value:
                    values += v.split(',')
            for v in values:
                if not v:
                    continue
                hiddens.append(v)
                selects.append(v)
                if self.model:
                    try:
                        item = self.model.objects.get(pk=v)
                        selects[-1] = unicode(item)
                        selected = item.pk
                    except (self.model.DoesNotExist, ValueError):
                        selects.pop()
                        hiddens.pop()
            if hiddens and selects:
                attrs_hidden['value'] = hiddens[0]
                attrs_select['value'] = selects[0]
        if 'id' not in self.attrs:
            attrs_hidden['id'] = 'id_%s' % name
            attrs_select['id'] = 'id_select_%s' % name
        if 'class' not in attrs_select:
            attrs_select['class'] = 'autocomplete'
        new = ''
        dct = {'attrs_select': mark_safe(flatatt(attrs_select)),
               'attrs_hidden': mark_safe(flatatt(attrs_hidden)),
               'name': name,
               'js': self.render_js(name, selected),
               'new': mark_safe(new)}
        html = loader.get_template(self.html_template).render(Context(dct))
        return html


class JQueryJqGrid(forms.RadioSelect):
    COL_TPL = "{name:'%(idx)s', index:'%(idx)s', sortable:true}"

    # class Media:
    #    js = ['%s/js/i18n/grid.locale-%s.js' % (settings.STATIC_URL,
    #                                            settings.COUNTRY),
    #          '%s/js/jquery.jqGrid.min.js' % settings.STATIC_URL]
    #    css = {'all': ['%s/media/ui.jqgrid.css' % settings.STATIC_URL]}

    def __init__(self, source, form, associated_model, attrs={},
                 table_cols='TABLE_COLS', multiple=False, multiple_cols=[2],
                 new=False, new_message="", source_full=None,
                 multiple_select=False):
        self.source = source
        self.form = form
        self.attrs = attrs
        self.associated_model = associated_model
        self.table_cols = table_cols
        self.multiple = multiple
        self.multiple_select = multiple_select
        self.multiple_cols = multiple_cols
        self.new, self.new_message = new, new_message
        self.source_full = source_full

    def get_cols(self, python=False):
        jq_col_names, extra_cols = [], []
        col_labels = {}
        if hasattr(self.associated_model, self.table_cols + '_LBL'):
            col_labels = getattr(self.associated_model,
                                 self.table_cols + '_LBL')
        for col_names in getattr(self.associated_model, self.table_cols):
            field_verbose_names = []
            field_verbose_name, field_name = "", ""
            if type(col_names) not in (list, tuple):
                col_names = [col_names]
            for col_name in col_names:
                field = self.associated_model
                keys = col_name.split('__')
                if '.' in col_name:
                    keys = col_name.split('.')
                f_name = ''
                for key in keys:
                    if hasattr(field, 'rel') and field.rel:
                        field = field.rel.to
                    try:
                        field = field._meta.get_field(key)
                        field_verbose_name = field.verbose_name
                        f_name = field.name
                    except (fields.FieldDoesNotExist, AttributeError):
                        if hasattr(field, key + '_lbl'):
                            f_name = key
                            field_verbose_name = getattr(field, key + '_lbl')
                        else:
                            continue
                if field_name:
                    field_name += "__"
                field_name += f_name
                field_verbose_names.append(unicode(field_verbose_name))
            if field_name in col_labels:
                jq_col_names.append(unicode(col_labels[field_name]))
            elif col_names and col_names[0] in col_labels:
                jq_col_names.append(unicode(col_labels[col_names[0]]))
            else:
                jq_col_names.append(settings.JOINT.join(
                    [f for f in field_verbose_names if f]))
            if not python:
                jq_col_names[-1] = u'"%s"' % jq_col_names[-1]
            if python:
                extra_cols.append(field_name)
            else:
                extra_cols.append(self.COL_TPL % {'idx': field_name})
        if not python:
            jq_col_names = jq_col_names and ", ".join(jq_col_names) or ""
            extra_cols = extra_cols and ", ".join(extra_cols) or ""
        return jq_col_names, extra_cols

    def render(self, name, value=None, attrs=None):
        t = loader.get_template('blocks/form_flex_snippet.html')
        form = self.form()
        rendered = t.render(Context({'form': form,
                                     'flex': True}))
        dct = {}
        if self.new:
            model_name = self.associated_model._meta.object_name.lower()
            dct['url_new'] = reverse('new-' + model_name, args=['0'])
            dct['new_message'] = self.new_message

        col_names, extra_cols = self.get_cols()

        col_idx = []
        for k in form.get_input_ids():
            col_idx.append(u'"%s"' % k)
        col_idx = col_idx and ", ".join(col_idx) or ""

        dct['encoding'] = settings.ENCODING or 'utf-8'
        dct['source'] = unicode(self.source)
        if unicode(self.source_full) and unicode(self.source_full) != 'None':
            dct['source_full'] = unicode(self.source_full)
        dct.update({'name': name,
                    'col_names': col_names,
                    'extra_cols': extra_cols,
                    'source': unicode(self.source),
                    'col_idx': col_idx,
                    'no_result': unicode(_("No results")),
                    'loading': unicode(_("Loading...")),
                    'remove': unicode(_(u"Remove")),
                    'sname': name.replace('-', ''),
                    'multiple': self.multiple,
                    'multiple_select': self.multiple_select,
                    'multi_cols': ",".join((u'"%d"' % col
                                           for col in self.multiple_cols))})
        t = loader.get_template('blocks/JQueryJqGrid.html')
        rendered += t.render(Context(dct))
        return mark_safe(rendered)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

import json

from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client

from ishtar_common.models import ImporterType, IshtarSiteProfile
from ishtar_common.tests import create_superuser
from archaeological_operations.tests import OperationInitTest, \
    ImportOperationTest
from archaeological_operations import models as models_ope
from archaeological_context_records import models
from ishtar_common import forms_common


class ImportContextRecordTest(ImportOperationTest):
    test_operations = False
    test_context_records = True

    fixtures = ImportOperationTest.fixtures + [
        settings.ROOT_PATH +
        '../archaeological_context_records/fixtures/initial_data-fr.json',
    ]

    def testMCCImportContextRecords(self, test=True):
        if test and not self.test_context_records:
            return
        self.testMCCImportParcels(test=False)

        old_nb = models.ContextRecord.objects.count()
        MCC = ImporterType.objects.get(name=u"MCC - UE")
        mcc_file = open(
            settings.ROOT_PATH +
            '../archaeological_context_records/tests/'
            'MCC-context-records-example.csv', 'rb')
        file_dict = {'imported_file': SimpleUploadedFile(mcc_file.name,
                                                         mcc_file.read())}
        post_dict = {'importer_type': MCC.pk, 'skip_lines': 1,
                     "encoding": 'utf-8'}
        form = forms_common.NewImportForm(data=post_dict, files=file_dict,
                                          instance=None)
        form.is_valid()
        if test:
            self.assertTrue(form.is_valid())
        impt = form.save(self.ishtar_user)
        impt.initialize()

        # doing manual connections
        hc = models.Unit.objects.get(txt_idx='not_in_context').pk
        self.setTargetKey('unit', 'hc', hc)
        self.setTargetKey('unit', 'hors-contexte', hc)
        layer = models.Unit.objects.get(txt_idx='layer').pk
        self.setTargetKey('unit', 'couche', layer)

        impt.importation()
        if not test:
            return

        # new ues has now been imported
        current_nb = models.ContextRecord.objects.count()
        self.assertTrue(current_nb == (old_nb + 4))
        self.assertEqual(
            models.ContextRecord.objects.filter(unit__pk=hc).count(), 3)
        self.assertEqual(
            models.ContextRecord.objects.filter(unit__pk=layer).count(), 1)


class ContextRecordInit(OperationInitTest):
    test_operations = False

    def create_context_record(self, user=None, data={}, force=False):
        if not getattr(self, 'context_records', None):
            self.context_records = []

        default = {'label': "Context record"}
        if force or not data.get('operation'):
            data['operation'] = self.get_default_operation(force=force)
        if not data.get('parcel') or not data['parcel'].pk:
            data['parcel'] = self.get_default_parcel()
        if not data.get('history_modifier'):
            data['history_modifier'] = self.get_default_user()

        default.update(data)
        self.context_records.append(models.ContextRecord.objects.create(
            **default))
        return self.context_records

    def get_default_context_record(self, force=False):
        return self.create_context_record(force=force)[0]

    def tearDown(self):
        if hasattr(self, 'context_records'):
            for cr in self.context_records:
                try:
                    cr.delete()
                except:
                    pass
            self.context_records = []
        super(ContextRecordInit, self).tearDown()


class ContextRecordTest(ContextRecordInit, TestCase):
    fixtures = ImportContextRecordTest.fixtures

    def setUp(self):
        IshtarSiteProfile.objects.create()
        self.username, self.password, self.user = create_superuser()
        self.create_context_record(data={"label": u"CR 1"})
        self.create_context_record(data={"label": u"CR 2"})

        cr_1 = self.context_records[0]
        cr_2 = self.context_records[1]
        sym_rel_type = models.RelationType.objects.filter(
            symmetrical=True).all()[0]
        self.cr_rel_type = sym_rel_type
        models.RecordRelations.objects.create(
            left_record=cr_1, right_record=cr_2, relation_type=sym_rel_type)

    def testSearchExport(self):
        c = Client()
        response = c.get(reverse('get-contextrecord'))
        # no result when no authentification
        self.assertTrue(not json.loads(response.content))
        c.login(username=self.username, password=self.password)
        response = c.get(reverse('get-contextrecord'))
        self.assertTrue(json.loads(response.content)['total'] == 2)
        # test search label
        response = c.get(reverse('get-contextrecord'),
                         {'label': 'cr 1'})
        self.assertEqual(json.loads(response.content)['total'], 1)
        # test search between relations
        response = c.get(reverse('get-contextrecord'),
                         {'label': 'cr 1',
                          'relation_types_0': self.cr_rel_type.pk})
        self.assertEqual(json.loads(response.content)['total'], 2)
        # test search between operation relations
        first_ope = self.operations[0]
        first_ope.year = 2010
        first_ope.save()
        cr_1 = self.context_records[0]
        cr_1.operation = first_ope
        cr_1.save()
        other_ope = self.operations[1]
        other_ope.year = 2016
        other_ope.save()
        cr_2 = self.context_records[1]
        cr_2.operation = other_ope
        cr_2.save()
        rel_ope = models_ope.RelationType.objects.create(
            symmetrical=True, label='Linked', txt_idx='link')
        models_ope.RecordRelations.objects.create(
            left_record=other_ope,
            right_record=first_ope,
            relation_type=rel_ope)
        response = c.get(reverse('get-contextrecord'),
                         {'operation__year': 2010,
                          'ope_relation_types_0': rel_ope.pk})
        self.assertEqual(json.loads(response.content)['total'], 2)
        # export
        response = c.get(reverse('get-contextrecord-full',
                                 kwargs={'type': 'csv'}), {'submited': '1'})
        # 2 lines + header
        lines = [line for line in response.content.split('\n') if line]
        self.assertEqual(len(lines), 3)

    def testZExternalID(self):
        cr = self.context_records[0]
        self.assertEqual(
            cr.external_id,
            u"{}-{}".format(cr.parcel.external_id, cr.label))


class RecordRelationsTest(ContextRecordInit, TestCase):
    fixtures = ImportOperationTest.fixtures
    model = models.ContextRecord

    def setUp(self):
        # two different context records
        self.create_context_record({"label": u"CR 1"})
        self.create_context_record({"label": u"CR 2"})

    def testRelations(self):
        sym_rel_type = models.RelationType.objects.create(
            symmetrical=True, txt_idx='sym')
        rel_type_1 = models.RelationType.objects.create(
            symmetrical=False, txt_idx='rel_1')
        # cannot be symmetrical and have an inverse_relation
        with self.assertRaises(ValidationError):
            rel_test = models.RelationType.objects.create(
                symmetrical=True, inverse_relation=rel_type_1, txt_idx='rel_3')
            rel_test.full_clean()
        # auto fill inverse relations
        rel_type_2 = models.RelationType.objects.create(
            symmetrical=False, inverse_relation=rel_type_1, txt_idx='rel_2')
        self.assertEqual(rel_type_1.inverse_relation, rel_type_2)

        cr_1 = self.context_records[0]
        cr_2 = self.context_records[1]

        # inserting a new symmetrical relation automatically creates the same
        # relation for the second context record
        rel = models.RecordRelations.objects.create(
            left_record=cr_1, right_record=cr_2, relation_type=sym_rel_type)
        self.assertEqual(models.RecordRelations.objects.filter(
            left_record=cr_2, right_record=cr_1,
            relation_type=sym_rel_type).count(), 1)

        # removing one symmetrical relation removes the other
        rel.delete()
        self.assertEqual(models.RecordRelations.objects.filter(
            left_record=cr_2, right_record=cr_1,
            relation_type=sym_rel_type).count(), 0)

        # for non-symmetrical relation, adding one relation automatically
        # adds the inverse
        rel = models.RecordRelations.objects.create(
            left_record=cr_1, right_record=cr_2, relation_type=rel_type_1)
        self.assertEqual(models.RecordRelations.objects.filter(
            left_record=cr_2, right_record=cr_1,
            relation_type=rel_type_2).count(), 1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf import settings
from django.contrib import admin

from ishtar_common.admin import HistorizedObjectAdmin, GeneralTypeAdmin

import models


class FileAdmin(HistorizedObjectAdmin):
    list_display = ['year', 'numeric_reference', 'internal_reference',
                    'end_date', 'file_type', 'general_contractor']
    if settings.COUNTRY == 'fr':
        list_display += ['saisine_type', 'permit_reference']
    list_filter = ("file_type", "year",)
    search_fields = ('towns__name',)
    model = models.File

admin.site.register(models.File, FileAdmin)

general_models = [models.FileType, models.PermitType]
if settings.COUNTRY == 'fr':
    general_models.append(models.SaisineType)
for model in general_models:
    admin.site.register(model, GeneralTypeAdmin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (C) 2013-2015 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

import locale
import re
from zipfile import ZipFile, ZIP_DEFLATED
from cStringIO import StringIO
from xml.etree.cElementTree import ElementTree, fromstring

from django.conf import settings
from ooo_translation import ooo_translation


def translate_context(context, locale):
    if locale not in ooo_translation:
        return context
    new_context = {}
    for k in context:
        new_key = k
        if k in ooo_translation[locale]:
            new_key = ooo_translation[locale][k]
        new_context[new_key] = context[k]
    return new_context

OOO_NS = "{urn:oasis:names:tc:opendocument:xmlns:text:1.0}"


def _set_value_from_formula(value, context, default_value):
    value = value.strip()
    if value.startswith("ooow:") and len(value) >= 5:
        value = value[5:]
    if value.startswith('"') and value.endswith('"') and len(value) > 1:
        value = value[1:-1]
    elif value in context:
        value = _format_value(context[value], default_value)
    else:
        value = None
    return value


def _parse_condition(condition, context, default_value):
    # parse only == and != operator
    operator = ""
    if "!=" in condition:
        operator = "!="
    elif "==" in condition:
        operator = "=="
    else:
        return
    var1, var2 = condition.split(operator)
    var1 = _set_value_from_formula(var1, context, default_value)
    var2 = _set_value_from_formula(var2, context, default_value)
    res = var1 == var2
    if operator == '!=':
        res = not res
    return res


def _format_value(value, default_value):
    if hasattr(value, 'strftime'):
        c_locale = settings.LANGUAGE_CODE.split('-')
        if len(c_locale) == 2:
            c_locale[1] = c_locale[1].upper()
        c_locale = "_".join(c_locale)
        if locale.getlocale()[0] != c_locale:
            for loc in (c_locale, c_locale + '.utf8'):
                try:
                    locale.setlocale(locale.LC_ALL, loc)
                    break
                except:
                    pass
        try:
            if settings.DATE_FORMAT:
                value = value.strftime(settings.DATE_FORMAT).lower()
            else:
                value = value.strftime('%x')
        except ValueError:
            value = unicode(value)
        if locale.getlocale()[1]:
            value = value.decode(locale.getlocale()[1])
    value = unicode(value) if value else default_value
    return value

VAR_EXPR = u"###%sVAR %s###"
WHOLE_KEY_FILTER = u"((?:(?: )*(?:<[^#>]*>)*(?: )*(?:[-a-zA-Z0-9_])*(?: )*)*)"
WHOLE_KEY_FILTER = u"([^#]*)"
RE_VAR = re.compile(VAR_EXPR % (WHOLE_KEY_FILTER, WHOLE_KEY_FILTER))
IF_EXPR = u"###%sIF %s###(.*)###ENDIF###"
RE_IF = re.compile(IF_EXPR % (WHOLE_KEY_FILTER, WHOLE_KEY_FILTER))
TAG_FILTER = re.compile(u"(<[^<^>]*>)")
KEY_FILTER = re.compile(u"([-a-zA-Z0-9_]*)")


def _filter_key(base_key):
    # return (key, extra_marker)
    # manage strange key such as:
    # test_<text:span text:style-name="T1">date</text:span>
    key = base_key[:]
    key = key.strip()
    tags, new_key = '', key[:]
    for tag in TAG_FILTER.findall(key):
        tags += tag
        new_key = new_key.replace(tag, '')
    full_key = ''
    for k in KEY_FILTER.findall(new_key):
        if not k:
            continue
        full_key += k
    return full_key, tags


def _custom_parsing(context, value, default_value=''):
    """
    ###VAR nom_var### for displaying a variable name
    ###IF nom_var### ###ENDIF### for conditionnal display
    Be carreful nested condition are not yet managed!
    """
    for regexp, sub_exp, if_cond in ((RE_IF, IF_EXPR, True),
                                     (RE_VAR, VAR_EXPR, False)):
        for base_key in regexp.findall(value[:]):
            v, val = "", None
            if if_cond:  # the value inside the if is parsed
                pre_tag, base_key, val = base_key
            else:
                pre_tag, base_key = base_key
            key, extra_markers = _filter_key(base_key)
            v = ''
            if pre_tag:
                v = pre_tag
            if key in context and context[key]:
                if if_cond:
                    v += _custom_parsing(context, val, default_value)
                else:
                    v += _format_value(context[key], default_value)
            # to preserve a consistent OOO file put extra_markers
            if extra_markers:
                v += extra_markers
            value = re.sub(sub_exp % (pre_tag, base_key), v, value)
    return value


def _ooo_replace(content, context, missing_keys, default_value=''):

    # regular ooo parsing
    for xp in ('variable-set', 'variable-get'):
        for p in content.findall(".//" + OOO_NS + xp):
            name = p.get(OOO_NS + "name")
            if name in context:
                value = context[name]
                p.text = _format_value(value, default_value)
            else:
                if default_value is not None:
                    p.text = default_value
                missing_keys.add(name)
    for p in content.findall(".//" + OOO_NS + "conditional-text"):
        condition = p.get(OOO_NS + "condition")
        res = 'true' if _parse_condition(condition, context, default_value) \
              else 'false'
        value = p.get(OOO_NS + 'string-value-if-' + res)
        value = _format_value(value, default_value)
        if value.strip() in context:
            value = context[value.strip()]
        p.text = value

    # raw content parsing
    str_io = StringIO()
    content.write(str_io)
    value = str_io.getvalue()
    value = _custom_parsing(context, value, default_value).encode('utf-8')
    return value


def ooo_replace(infile, outfile, context, default_value=''):
    inzip = ZipFile(infile, 'r', ZIP_DEFLATED)
    outzip = ZipFile(outfile, 'w', ZIP_DEFLATED)

    values = {}
    missing_keys = set()
    for xml_file in ('content.xml', 'styles.xml'):
        content = ElementTree(fromstring(inzip.read(xml_file)))
        values[xml_file] = _ooo_replace(content, context, missing_keys,
                                        default_value)

    for f in inzip.infolist():
        if f.filename in values:
            outzip.writestr(f.filename, values[f.filename])
        else:
            outzip.writestr(f, inzip.read(f.filename))

    inzip.close()
    outzip.close()
    return missing_keys

if __name__ == '__main__':
    infile = "../archaeological_files/tests/"\
             "AR_dossier_DRAC_modele_ishtar_1-MOD.odt"
    outfile = "../archaeological_files/tests/"\
              "AR_dossier_DRAC_modele_ishtar-test.odt"
    rep = {"file_incharge_surname": u"Yann",
           "file_incharge_name": u"Le Jeune",
           "fileact_ref": u"ref"}
    ooo_replace(infile, outfile, rep, default_value="")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2013-2014 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

import sys

from django.conf import settings
from django.db.models import Q

from ishtar_common.models import Town, Department

def load_towns():
    from geodjangofla.models import Commune
    q = None
    for dpt_number in settings.ISHTAR_DPTS:
        query = Q(insee_com__istartswith=dpt_number)
        if q:
            q = q | query
        else:
            q = query
    if q:
        q = Commune.objects.filter(q)
    else:
        q = Commune.objects
    nb, updated = 0, 0
    for town in q.all():
        surface = town.superficie or 0
        surface = surface * 10000
        defaults = {'name':town.nom_comm, 'surface':surface,
                    'center':town.centroid}
        town, created = Town.objects.get_or_create(numero_insee=town.insee_com,
                                   defaults=defaults)
        if created:
            nb += 1
        else:
            updated += 1
            for k in defaults:
                setattr(town, k, defaults[k])
            town.save()
    return nb, updated

def update_towns():
    nb, updated = 0, 0
    dpts = dict([(dpt.number, dpt) for dpt in Department.objects.all()])
    q = Town.objects.filter(numero_insee__isnull=False)
    total = q.count()
    for idx, town in enumerate(q.all()):
        sys.stdout.write('\rProcessing... %s/%d' % (
                                str(idx+1).zfill(len(str(total))), total))
        if len(town.numero_insee) < 2:
            continue
        dpt_code = town.numero_insee[:2]
        if dpt_code.startswith('9') and int(dpt_code) > 95:
            dpt_code = town.numero_insee[:3]
        if dpt_code not in dpts:
            sys.stdout.write('Missing department with INSEE code: %s' % dpt_code)
            continue
        if town.departement == dpts[dpt_code]:
            continue
        if town.departement:
            updated += 1
        else:
            nb += 1
        town.departement = dpts[dpt_code]
        town.save()
    sys.stdout.write('\n')
    return nb, updated



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012-2015 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.contrib import admin

from ishtar_common.admin import HistorizedObjectAdmin, GeneralTypeAdmin

import models


class DatingAdmin(admin.ModelAdmin):
    list_display = ('period', 'start_date', 'end_date', 'dating_type',
                    'quality')
    list_filter = ("period", 'dating_type', 'quality')
    model = models.Dating

admin.site.register(models.Dating, DatingAdmin)


class ContextRecordAdmin(HistorizedObjectAdmin):
    list_display = ('label', 'length', 'width',
                    'thickness', 'depth')
    list_filter = ('has_furniture',)
    search_fields = ('parcel__operation__name', "datings__period__label")
    model = models.ContextRecord

admin.site.register(models.ContextRecord, ContextRecordAdmin)


class ContextRecordSourceAdmin(admin.ModelAdmin):
    list_display = ('context_record', 'title', 'source_type',)
    list_filter = ('source_type',)
    search_fields = ('title', )
    model = models.ContextRecordSource

admin.site.register(models.ContextRecordSource, ContextRecordSourceAdmin)


class RecordRelationsAdmin(admin.ModelAdmin):
    list_display = ('left_record', 'relation_type', 'right_record')
    list_filter = ('relation_type',)

admin.site.register(models.RecordRelations, RecordRelationsAdmin)


class RelationTypeAdmin(admin.ModelAdmin):
    list_display = ('label', 'txt_idx', 'tiny_label', 'available',
                    'symmetrical', 'inverse_relation')

admin.site.register(models.RelationType, RelationTypeAdmin)

general_models = [models.DatingType, models.DatingQuality,
                  models.Unit, models.ActivityType, models.IdentificationType]
for model in general_models:
    admin.site.register(model, GeneralTypeAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf import settings
from django.conf.urls.defaults import patterns, include, url
from django.conf.urls.static import static

from menus import menu

from ishtar_common import views
from ishtar_common.wizards import check_rights

# be carreful: each check_rights must be relevant with ishtar_menu

# forms
urlpatterns = patterns(
    '',
    # internationalization
    url(r'^i18n/', include('django.conf.urls.i18n')),
    # General
    url(r'shortcut_menu/', views.shortcut_menu, name='shortcut-menu'),
    url(r'display/(?P<item_type>\w+)/(?P<pk>\d+)/',
        views.DisplayItemView.as_view(), name='display-item'),
    url(r'person_search/(?P<step>.+)?$',
        check_rights(['add_person'])(
            views.person_search_wizard), name='person_search'),
    url(r'person_creation/(?P<step>.+)?$',
        check_rights(['add_person'])(
            views.person_creation_wizard), name='person_creation'),
    url(r'person_modification/(?P<step>.+)?$',
        check_rights(['change_person', 'change_own_person'])(
            views.person_modification_wizard), name='person_modification'),
    url(r'person_modify/(?P<pk>.+)/$', views.person_modify,
        name='person_modify'),
    url(r'person_deletion/(?P<step>.+)?$',
        check_rights(['change_person', 'change_own_person'])(
            views.person_deletion_wizard), name='person_deletion'),
    url(r'^person-edit/$',
        check_rights(['add_person'])(
            views.PersonCreate.as_view()), name='person_create'),
    url(r'^person-edit/(?P<pk>\d+)$',
        check_rights(['change_person', 'change_own_person'])(
            views.PersonEdit.as_view()), name='person_edit'),
    url(r'organization_search/(?P<step>.+)?$',
        check_rights(['add_organization'])(
            views.organization_search_wizard), name='organization_search'),
    url(r'organization_creation/(?P<step>.+)?$',
        check_rights(['add_organization'])(
            views.organization_creation_wizard), name='organization_creation'),
    url(r'organization_modification/(?P<step>.+)?$',
        check_rights(['change_organization', 'change_own_organization'])(
            views.organization_modification_wizard),
        name='organization_modification'),
    url(r'organization_modify/(?P<pk>.+)/$', views.organization_modify,
        name='organization_modify'),
    url(r'organization_deletion/(?P<step>.+)?$',
        check_rights(['change_organization', 'change_own_organization'])(
            views.organization_deletion_wizard), name='organization_deletion'),
    url(r'organization-edit/$',
        check_rights(['add_organization'])(
            views.OrganizationCreate.as_view()), name='organization_create'),
    url(r'organization-edit/(?P<pk>\d+)$',
        check_rights(['change_organization', 'change_own_organization'])(
            views.OrganizationEdit.as_view()), name='organization_edit'),
    url(r'organization-person-edit/$',
        check_rights(['add_organization'])(
            views.OrganizationPersonCreate.as_view()),
        name='organization_person_create'),
    url(r'organization-person-edit/(?P<pk>\d+)$',
        check_rights(['change_organization', 'change_own_organization'])(
            views.OrganizationPersonEdit.as_view()),
        name='organization_person_edit'),
    url(r'get-ishtaruser/(?P<type>.+)?$',
        views.get_ishtaruser,
        name='get-ishtaruser'),
    url(r'account_management/(?P<step>.+)?$',
        check_rights(['add_ishtaruser'])(
            views.account_management_wizard), name='account_management'),
    url(r'account_deletion/(?P<step>.+)?$',
        check_rights(['add_ishtaruser'])(
            views.account_deletion_wizard), name='account_deletion'),
    url(r'^import-new/$',
        check_rights(['change_import'])(
            views.NewImportView.as_view()), name='new_import'),
    url(r'^import-list/$',
        check_rights(['change_import'])(
            views.ImportListView.as_view()),
        name='current_imports'),
    url(r'^import-list-old/$',
        check_rights(['change_import'])(
            views.ImportOldListView.as_view()),
        name='old_imports'),
    url(r'^import-delete/(?P<pk>[0-9]+)/$',
        views.ImportDeleteView.as_view(), name='import_delete'),
    url(r'^import-link-unmatched/(?P<pk>[0-9]+)/$',
        views.ImportLinkView.as_view(), name='import_link_unmatched'),
)

actions = []
for section in menu.childs:
    for menu_item in section.childs:
        if hasattr(menu_item, 'childs'):
            for menu_subitem in menu_item.childs:
                actions.append(menu_subitem.idx)
        else:
            actions.append(menu_item.idx)
actions = r"|".join(actions)

# other views
urlpatterns += patterns(
    'ishtar_common.views',
    # General
    url(r'dashboard-main/$', 'dashboard_main',
        name='dashboard-main'),
    url(r'dashboard-main/(?P<item_name>[a-z-]+)/$', 'dashboard_main_detail',
        name='dashboard-main-detail'),
    url(r'update-current-item/$', 'update_current_item',
        name='update-current-item'),
    url(r'pin/(?P<item_type>[a-z-]+)/(?P<pk>\d+)/$', 'update_current_item',
        name='pin'),
    url(r'unpin/(?P<item_type>[a-z-]+)/$', 'unpin', name='unpin'),
    url(r'new-person/(?:(?P<parent_name>[^/]+)/)?(?:(?P<limits>[^/]+)/)?$',
        'new_person', name='new-person'),
    url(r'new-person-noorga/'
        r'(?:(?P<parent_name>[^/]+)/)?(?:(?P<limits>[^/]+)/)?$',
        'new_person_noorga', name='new-person-noorga'),
    url(r'autocomplete-person(?:/([0-9_]+))?(?:/([0-9_]*))?/(user)?$',
        'autocomplete_person', name='autocomplete-person'),
    url(r'autocomplete-person-permissive(?:/([0-9_]+))?(?:/([0-9_]*))?'
        r'/(user)?$', 'autocomplete_person_permissive',
        name='autocomplete-person-permissive'),
    url(r'get-person/(?P<type>.+)?$', 'get_person',
        name='get-person'),
    url(r'get-person-full/(?P<type>.+)?$', 'get_person',
        name='get-person-full', kwargs={'full': True}),
    url(r'show-person(?:/(?P<pk>.+))?/(?P<type>.+)?$',
        'show_person', name='show-person'),
    url(r'department-by-state/(?P<state_id>.+)?$', 'department_by_state',
        name='department-by-state'),
    url(r'autocomplete-town/?$', 'autocomplete_town',
        name='autocomplete-town'),
    url(r'autocomplete-advanced-town/(?P<department_id>[0-9]+[ABab]?)?$',
        'autocomplete_advanced_town', name='autocomplete-advanced-town'),
    url(r'autocomplete-department/?$', 'autocomplete_department',
        name='autocomplete-department'),
    url(r'new-author/(?:(?P<parent_name>[^/]+)/)?(?:(?P<limits>[^/]+)/)?$',
        'new_author', name='new-author'),
    url(r'autocomplete-author/$', 'autocomplete_author',
        name='autocomplete-author'),
    url(r'new-organization/(?:(?P<parent_name>[^/]+)/)?'
        r'(?:(?P<limits>[^/]+)/)?$',
        'new_organization', name='new-organization'),
    url(r'get-organization/(?P<type>.+)?$', 'get_organization',
        name='get-organization'),
    url(r'get-organization-full/(?P<type>.+)?$', 'get_organization',
        name='get-organization-full', kwargs={'full': True}),
    url(r'show-organization(?:/(?P<pk>.+))?/(?P<type>.+)?$',
        'show_organization', name='show-organization'),
    url(r'autocomplete-organization/([0-9_]+)?$',
        'autocomplete_organization', name='autocomplete-organization'),
    url(r'admin-globalvar/', views.GlobalVarEdit.as_view(),
        name='admin-globalvar'),
    url(r'person-merge/(?:(?P<page>\d+)/)?$', 'person_merge',
        name='person_merge'),
    url(r'person-manual-merge/$',
        views.PersonManualMerge.as_view(),
        name='person_manual_merge'),
    url(r'person-manual-merge-items/(?P<pks>[0-9_]+?)/$',
        views.PersonManualMergeItems.as_view(),
        name='person_manual_merge_items'),
    url(r'organization-merge/(?:(?P<page>\d+)/)?$', 'organization_merge',
        name='organization_merge'),
    url(r'orga-manual-merge/$', views.OrgaManualMerge.as_view(),
        name='orga_manual_merge'),
    url(r'orga-manual-merge-items/(?P<pks>[0-9_]+?)/$',
        views.OrgaManualMergeItems.as_view(),
        name='orga_manual_merge_items'),
    url(r'reset/$', 'reset_wizards', name='reset_wizards'),
    url(r'activate-all-search/$', 'activate_all_search',
        name='activate-all-search'),
    url(r'activate-own-search/$', 'activate_own_search',
        name='activate-own-search'),
    url(r'activate-advanced-menu/$', 'activate_advanced_shortcut_menu',
        name='activate-advanced-menu'),
    url(r'activate-simple-menu/$', 'activate_simple_shortcut_menu',
        name='activate-simple-menu'),
    url(r'(?P<action_slug>' + actions + r')/$', 'action', name='action'),
)

if settings.DEBUG:
    urlpatterns += static(settings.STATIC_URL,
                          document_root=settings.STATIC_ROOT)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

import json

from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import HttpResponse
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _

from ishtar_common.forms_common import AuthorFormset, SourceForm
from ishtar_common.views import get_item, show_item, revert_item
from ishtar_common.wizards import SearchWizard
from wizards import *
from forms import *
import models

show_contextrecord = show_item(models.ContextRecord, 'contextrecord')
revert_contextrecord = revert_item(models.ContextRecord)

contextrecord_extra_keys = {
    'parcel__town': 'parcel__town__pk',
    'operation__year': 'operation__year__contains',
    'operation__code_patriarche': 'operation__code_patriarche',
    'operation__operation_code': 'operation__operation_code',
    'datings__period': 'datings__period__pk',
    'parcel_0': 'operation__parcels__section',
    'parcel_1': 'operation__parcels__parcel_number',
    'parcel_2': 'operation__parcels__public_domain',
    'label': 'label__icontains',
    'archaeological_sites': 'operation__archaeological_sites__pk',
    'cached_label': 'cached_label__icontains',
}


def autocomplete_contextrecord(request):
    if (not request.user.has_perm(
            'archaeological_context_records.view_contextrecord',
            models.ContextRecord)
        and not request.user.has_perm(
            'archaeological_context_records.view_own_contextrecord',
            models.ArchaeologicalSite)):
        return HttpResponse(mimetype='text/plain')
    if not request.GET.get('term'):
        return HttpResponse(mimetype='text/plain')
    q = request.GET.get('term')
    query = Q()
    if request.GET.get('operation__pk'):
        query = Q(operation__pk=request.GET.get('operation__pk'))
    for q in q.split(' '):
        qt = Q(parcel__section__icontains=q) | \
            Q(parcel__parcel_number__icontains=q) | \
            Q(label__icontains=q)
        query = query & qt
    limit = 15
    items = models.ContextRecord.objects\
        .filter(query).order_by('parcel__section', 'parcel__parcel_number',
                                'label')[:limit]
    data = json.dumps([{'id': item.pk,
                        'value': unicode(item)[:60]}
                       for item in items])
    return HttpResponse(data, mimetype='text/plain')

get_contextrecord = get_item(
    models.ContextRecord,
    'get_contextrecord', 'contextrecord',
    relation_types_prefix={'ope_relation_types': 'operation__'},
    relative_session_names=[
        ('operation', 'operation__pk'),
        ('file', 'operation__associated_file__pk')],
    extra_request_keys=contextrecord_extra_keys,)
get_contextrecord_for_ope = get_item(
    models.ContextRecord,
    'get_contextrecord', 'contextrecord',
    extra_request_keys=contextrecord_extra_keys,
    own_table_cols=models.ContextRecord.TABLE_COLS_FOR_OPE)

show_contextrecordsource = show_item(models.ContextRecordSource,
                                     'contextrecordsource')

get_contextrecordsource = get_item(
    models.ContextRecordSource,
    'get_contextrecordsource', 'contextrecordsource',
    relative_session_names=[
        ('contextrecord', 'context_record__pk'),
        ('operation', 'context_record__operation__pk'),
        ('file', 'context_record__operation__associated_file__pk')],
    bool_fields=['duplicate'],
    extra_request_keys={
        'title': 'title__icontains',
        'description': 'description__icontains',
        'comment': 'comment__icontains',
        'person': 'authors__person__pk',
        'additional_information': 'additional_information__icontains',
        'context_record__parcel__town': 'context_record__parcel__town__pk',
        'context_record__operation__year': 'context_record__operation__year',
        'context_record__operation__operation_code':
        'context_record__operation__operation_code',
        'context_record__operation__code_patriarche':
        'context_record__operation__code_patriarche',
        'context_record__operation': 'context_record__operation__pk',
        'context_record__datings__period':
            'context_record__datings__period__pk',
        'context_record__unit': 'context_record__unit__pk',
    })

get_contextrecordrelation = get_item(
    models.RecordRelations, 'get_contextrecordrelation',
    'contextrecordrelation',
    extra_request_keys={
        "left_record__operation": "left_record__operation__pk"
    })

record_search_wizard = SearchWizard.as_view([
    ('general-record_search', RecordFormSelection)],
    label=_(u"Context record search"),
    url_name='record_search',)

record_creation_wizard = RecordWizard.as_view([
    ('selec-record_creation', OperationRecordFormSelection),
    ('general-record_creation', RecordFormGeneral),
    ('datings-record_creation', DatingFormSet),
    ('interpretation-record_creation', RecordFormInterpretation),
    ('relations-record_creation', RecordRelationsFormSet),
    ('final-record_creation', FinalForm)],
    label=_(u"New context record"),
    url_name='record_creation',)

record_modification_wizard = RecordModifWizard.as_view([
    ('selec-record_modification', RecordFormSelection),
    ('operation-record_modification', OperationFormSelection),
    ('general-record_modification', RecordFormGeneral),
    ('datings-record_modification', DatingFormSet),
    ('interpretation-record_modification', RecordFormInterpretation),
    ('relations-record_modification', RecordRelationsFormSet),
    ('final-record_modification', FinalForm)],
    label=_(u"Context record modification"),
    url_name='record_modification',)


def record_modify(request, pk):
    record_modification_wizard(request)
    RecordModifWizard.session_set_value(
        request, 'selec-record_modification', 'pk', pk, reset=True)
    return redirect(reverse('record_modification',
                    kwargs={'step': 'operation-record_modification'}))

record_deletion_wizard = RecordDeletionWizard.as_view([
    ('selec-record_deletion', RecordFormSelection),
    ('final-record_deletion', RecordDeletionForm)],
    label=_(u"Context record deletion"),
    url_name='record_deletion',)

record_source_search_wizard = SearchWizard.as_view([
    ('selec-record_source_search', RecordSourceFormSelection)],
    label=_(u"Context record: source search"),
    url_name='record_source_search',)

record_source_creation_wizard = RecordSourceWizard.as_view([
    ('selec-record_source_creation', SourceRecordFormSelection),
    ('source-record_source_creation', SourceForm),
    ('authors-record_source_creation', AuthorFormset),
    ('final-record_source_creation', FinalForm)],
    label=_(u"Context record: new source"),
    url_name='record_source_creation',)

record_source_modification_wizard = RecordSourceWizard.as_view([
    ('selec-record_source_modification', RecordSourceFormSelection),
    ('source-record_source_modification', SourceForm),
    ('authors-record_source_modification', AuthorFormset),
    ('final-record_source_modification', FinalForm)],
    label=_(u"Context record: source modification"),
    url_name='record_source_modification',)


def record_source_modify(request, pk):
    record_source_modification_wizard(request)
    RecordSourceWizard.session_set_value(
        request, 'selec-record_source_modification', 'pk', pk, reset=True)
    return redirect(reverse(
        'record_source_modification',
        kwargs={'step': 'source-record_source_modification'}))

record_source_deletion_wizard = RecordSourceDeletionWizard.as_view([
    ('selec-record_source_deletion', RecordSourceFormSelection),
    ('final-record_source_deletion', RecordDeletionForm)],
    label=_(u"Context record: source deletion"),
    url_name='record_source_deletion',)
